@@ -1,4 +1,4 @@
-/* $Id: cpl_array_impl.h,v 1.5 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_array_impl.h,v 1.6 2013-04-26 14:25:54 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.5 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-04-26 14:25:54 $
+ * $Revision: 1.6 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifndef CPL_ARRAY_IMPL_H
@@ -35,7 +35,7 @@ CPL_BEGIN_DECLS
 
 cpl_column *cpl_array_get_column(cpl_array *);
 const cpl_column *cpl_array_get_column_const(const cpl_array *);
-cpl_error_code cpl_array_set_column(cpl_array *, const cpl_column *);
+cpl_error_code cpl_array_set_column(cpl_array *, cpl_column *);
 
 CPL_END_DECLS
 
