@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic_body.h,v 1.100 2012/12/13 11:41:39 llundin Exp $
+/* $Id: cpl_image_basic_body.h,v 1.107 2013-10-22 11:50:33 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -18,7 +18,58 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#if defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_OPERATE
+#if defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_HYPOT
+
+/* Declaration */
+static cpl_error_code CPL_CONCAT2X(cpl_image_hypot, CPL_CONCAT2X(CPL_TYPE1,
+                                   CPL_CONCAT2X(CPL_TYPE2, CPL_TYPE3)))
+    (cpl_image *,
+     const cpl_image *,
+     const cpl_image *)
+#ifdef CPL_HAVE_ATTR_NONNULL
+    __attribute__((nonnull(1, 3)))
+#endif
+;
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Type-specific version of the function
+  @param    self     Pre-allocated image to hold the result
+  @param    first    First operand (may be aliased to self)
+  @param    second   Second operand
+  @return   CPL_ERROR_NONE or the relevant the #_cpl_error_code_ on error
+  @see      cpl_image_hypot()
+  
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_error_code CPL_CONCAT2X(cpl_image_hypot, CPL_CONCAT2X(CPL_TYPE1,
+                                   CPL_CONCAT2X(CPL_TYPE2, CPL_TYPE3)))
+    (cpl_image * self,
+     const cpl_image * first,
+     const cpl_image * second)
+{
+
+    CPL_TYPE1       * p1 = (      CPL_TYPE1 *)self->pixels;
+    const CPL_TYPE3 * p3 = (const CPL_TYPE3 *)second->pixels;
+    const size_t     nxy = (size_t)(self->nx * self->ny);
+    size_t           i;
+
+    if (first == NULL) {
+        for (i = 0; i < nxy; i++)
+            p1[i] = (CPL_TYPE1)CPL_HYPOT(p1[i], p3[i]);
+    } else {
+        const CPL_TYPE2 * p2  = (const CPL_TYPE2 *)first->pixels;
+
+        for (i = 0; i < nxy; i++)
+            p1[i] = (CPL_TYPE1)CPL_HYPOT(p2[i], p3[i]);
+    }
+
+    return CPL_ERROR_NONE;
+}
+
+
+#elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_OPERATE
 case CPL_TYPE_T: {
     const CPL_TYPE * p2  = (const CPL_TYPE *)image2->pixels;
     const size_t     nxy = (size_t)(image1->nx * image1->ny);
@@ -81,15 +132,13 @@ case CPL_TYPE_T: {
 #define CPL_TYPE_CONCAT double
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_MATH_ABS fabs
-#define CPL_MATH_TYPE double
 #define CPL_TYPE_IS_FPOINT
 
 #elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_FLOAT
 #define CPL_TYPE float
 #define CPL_TYPE_CONCAT float
 #define CPL_TYPE_T CPL_TYPE_FLOAT
-#define CPL_MATH_ABS fabs
-#define CPL_MATH_TYPE double
+#define CPL_MATH_ABS fabsf
 #define CPL_TYPE_IS_FPOINT
 
 #elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_INT
@@ -97,17 +146,18 @@ case CPL_TYPE_T: {
 #define CPL_TYPE_CONCAT int
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_MATH_ABS abs
-#define CPL_MATH_TYPE int
 
 #elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_DOUBLE_COMPLEX
 #define CPL_TYPE double complex
 #define CPL_TYPE_CONCAT double_complex
+#define CPL_MATH_ABS cabs
 #define CPL_TYPE_T CPL_TYPE_DOUBLE_COMPLEX
 #define CPL_TYPE_IS_FPOINT
 
 #elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_FLOAT_COMPLEX
 #define CPL_TYPE float complex
 #define CPL_TYPE_CONCAT float_complex
+#define CPL_MATH_ABS cabsf
 #define CPL_TYPE_T CPL_TYPE_FLOAT_COMPLEX
 #define CPL_TYPE_IS_FPOINT
 
@@ -115,7 +165,6 @@ case CPL_TYPE_T: {
 #undef CPL_TYPE
 #undef CPL_TYPE_T
 #undef CPL_MATH_ABS
-#undef CPL_MATH_TYPE
 #undef CPL_TYPE_ADD
 #endif
 
@@ -359,7 +408,53 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_power)(cpl_image * self,
     /* In spite of checks pow() may still overflow */
     errno = 0;
 
-    if (exponent < 0.0) {
+    if (exponent == -0.5) {
+        /* Pixel value must be positive */
+        for (i = 0; i < n; i++) {
+            if (!cbpm || !cbpm[i]) { /* Pixel is not bad (yet) */
+                if ((double)data[i] > 0.0) {
+                    data[i] = (CPL_TYPE)(1.0/sqrt((double)data[i]));
+                } else {
+                    errno = EDOM; /* Use errno to reuse code */
+                }
+                if (errno) {
+                    errno = 0;
+                    if (!bpm) {
+                        /* Create the Bad Pixel Map if necessary */
+                        /* If cbpm is NULL, then keep it that way to save
+                           some checking */
+                        bpm = cpl_mask_get_data(cpl_image_get_bpm(self));
+                    }
+                    bpm[i] = CPL_BINARY_1;
+                    data[i] = 0; /* Set bad pixel to zero */
+                }
+            }
+        }
+#ifdef HAVE_CBRT
+    } else if (exponent == -1.0 / 3.0) {
+        /* Pixel value must be positive */
+        for (i = 0; i < n; i++) {
+            if (!cbpm || !cbpm[i]) { /* Pixel is not bad (yet) */
+                if ((double)data[i] > 0.0) {
+                    data[i] = (CPL_TYPE)(1.0/cbrt((double)data[i]));
+                } else {
+                    errno = EDOM; /* Use errno to reuse code */
+                }
+                if (errno) {
+                    errno = 0;
+                    if (!bpm) {
+                        /* Create the Bad Pixel Map if necessary */
+                        /* If cbpm is NULL, then keep it that way to save
+                           some checking */
+                        bpm = cpl_mask_get_data(cpl_image_get_bpm(self));
+                    }
+                    bpm[i] = CPL_BINARY_1;
+                    data[i] = 0; /* Set bad pixel to zero */
+                }
+            }
+        }
+#endif
+    } else if (exponent < 0.0) {
         if (exponent != ceil(exponent)) {
             /* Pixel value must be positive */
             for (i = 0; i < n; i++) {
@@ -628,13 +723,42 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     cpl_image * self;
 
-    cpl_ensure(image1     != NULL,       CPL_ERROR_NULL_INPUT,    NULL);
-    cpl_ensure(image2     != NULL,       CPL_ERROR_NULL_INPUT,    NULL);
-    cpl_ensure(image1->nx == image2->nx, CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
-    cpl_ensure(image1->ny == image2->ny, CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+    if (image1 == NULL) {
+        (void)cpl_error_set_message_(CPL_ERROR_NULL_INPUT,
+                                     "First image is NULL");
+        return NULL;
+    } else if (image2 == NULL) {
+        (void)cpl_error_set_message_(CPL_ERROR_NULL_INPUT,
+                                     "Second image is NULL");
+        return NULL;
+    } else if (image1->nx != image2->nx) {
+        (void)cpl_error_set_message_(CPL_ERROR_INCOMPATIBLE_INPUT,
+                                     "Images are %" CPL_SIZE_FORMAT " X %"
+                                     CPL_SIZE_FORMAT " (%s) <=> %"
+                                     CPL_SIZE_FORMAT " X %" CPL_SIZE_FORMAT
+                                     " (%s)", image1->nx, image1->ny,
+                                     cpl_type_get_name(image1->type),
+                                     image2->nx, image2->ny,
+                                     cpl_type_get_name(image2->type));
+        return NULL;
+    } else if (image1->ny != image2->ny) {
+        (void)cpl_error_set_message_(CPL_ERROR_INCOMPATIBLE_INPUT,
+                                     "Images are %" CPL_SIZE_FORMAT " X %"
+                                     CPL_SIZE_FORMAT " (%s) <=> %"
+                                     CPL_SIZE_FORMAT " X %" CPL_SIZE_FORMAT
+                                     " (%s)", image1->nx, image1->ny,
+                                     cpl_type_get_name(image1->type),
+                                     image2->nx, image2->ny,
+                                     cpl_type_get_name(image2->type));
+        return NULL;
+    }
+
+    assert( image1->pixels );
+    assert( image2->pixels );
 
     /* Switch on the first passed image type */
     switch (image1->type) {
+#define CPL_MATH_ABS1 abs
     case CPL_TYPE_INT: {
         const int * p1 = (const int *)image1->pixels;
         int       * pout = (int *)cpl_malloc(image1->nx * image1->ny *
@@ -644,21 +768,27 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch (image2->type) {
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_TYPE   int
+#define CPL_MATH_ABS2 abs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT
 #define CPL_TYPE   float
+#define CPL_MATH_ABS2 fabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_TYPE   double
+#define CPL_MATH_ABS2 fabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
         default:
             cpl_free(pout);
@@ -668,6 +798,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         self = pout ? cpl_image_wrap_int(image1->nx, image1->ny, pout) : NULL;
         break;
     }
+#undef CPL_MATH_ABS1
+#define CPL_MATH_ABS1 fabsf
     case CPL_TYPE_FLOAT: {
         const float * p1   = (const float *)image1->pixels;
         float       * pout = (float *)cpl_malloc(image1->nx * image1->ny *
@@ -677,21 +809,27 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch (image2->type) {
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_TYPE   int
+#define CPL_MATH_ABS2 abs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT
 #define CPL_TYPE   float
+#define CPL_MATH_ABS2 fabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_TYPE   double
+#define CPL_MATH_ABS2 fabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
         default:
             cpl_free(pout);
@@ -701,6 +839,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         self = pout ? cpl_image_wrap_float(image1->nx, image1->ny, pout) : NULL;
         break;
     }
+#undef CPL_MATH_ABS1
+#define CPL_MATH_ABS1 fabs
     case CPL_TYPE_DOUBLE: {
         const double * p1   = (const double *)image1->pixels;
         double       * pout = (double *)cpl_malloc(image1->nx * image1->ny *
@@ -710,21 +850,27 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch (image2->type) {
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_TYPE   int
+#define CPL_MATH_ABS2 abs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT
 #define CPL_TYPE   float
+#define CPL_MATH_ABS2 fabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_TYPE   double
+#define CPL_MATH_ABS2 fabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
         default:
             cpl_free(pout);
@@ -735,6 +881,9 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             : NULL;
         break;
     }
+
+#undef CPL_MATH_ABS1
+#define CPL_MATH_ABS1 cabsf
     case CPL_TYPE_FLOAT_COMPLEX: {
         const float complex * p1   = (const float complex *)image1->pixels;
         float complex       * pout =
@@ -744,33 +893,43 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch (image2->type) {
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_TYPE   int
+#define CPL_MATH_ABS2 abs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT
 #define CPL_TYPE   float
+#define CPL_MATH_ABS2 fabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_TYPE   double
+#define CPL_MATH_ABS2 fabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT_COMPLEX
 #define CPL_TYPE   float complex
+#define CPL_MATH_ABS2 cabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE_COMPLEX
 #define CPL_TYPE   double complex
+#define CPL_MATH_ABS2 cabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
         default:
             cpl_free(pout);
@@ -781,6 +940,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             : NULL;
         break;
     }
+#undef CPL_MATH_ABS1
+#define CPL_MATH_ABS1 cabs
     case CPL_TYPE_DOUBLE_COMPLEX: {
         const double complex * p1   = (const double complex *)image1->pixels;
         double complex       * pout =
@@ -790,33 +951,43 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch (image2->type) {
 #define CPL_TYPE_T CPL_TYPE_INT
 #define CPL_TYPE   int
+#define CPL_MATH_ABS2 abs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT
 #define CPL_TYPE   float
+#define CPL_MATH_ABS2 fabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE
 #define CPL_TYPE   double
+#define CPL_MATH_ABS2 fabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_FLOAT_COMPLEX
 #define CPL_TYPE   float complex
+#define CPL_MATH_ABS2 cabsf
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
 #define CPL_TYPE_T CPL_TYPE_DOUBLE_COMPLEX
 #define CPL_TYPE   double complex
+#define CPL_MATH_ABS2 cabs
 #include "cpl_image_basic_body.h"
 #undef CPL_TYPE_T
 #undef CPL_TYPE
+#undef CPL_MATH_ABS2
 
         default:
             cpl_free(pout);
@@ -827,6 +998,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                                                     pout) : NULL;
         break;
     }
+#undef CPL_MATH_ABS1
     default:
         (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
         self = NULL;
@@ -861,10 +1033,29 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 #undef CPL_OPERATION
 #define CPL_OPERATION CPL_IMAGE_BASIC_OPERATE_LOCAL
 
-    cpl_ensure_code(im1    != NULL,     CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(im2    != NULL,     CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(im1->nx == im2->nx, CPL_ERROR_ILLEGAL_INPUT);
-    cpl_ensure_code(im1->ny == im2->ny, CPL_ERROR_ILLEGAL_INPUT);
+    if (im1 == NULL) {
+        return cpl_error_set_message_(CPL_ERROR_NULL_INPUT,
+                                      "First image is NULL");
+    } else if (im2 == NULL) {
+        return cpl_error_set_message_(CPL_ERROR_NULL_INPUT,
+                                      "Second image is NULL");
+    } else if (im1->nx != im2->nx) {
+        return cpl_error_set_message_(CPL_ERROR_INCOMPATIBLE_INPUT,
+                                      "Images are %" CPL_SIZE_FORMAT " X %"
+                                      CPL_SIZE_FORMAT " (%s) <=> %"
+                                      CPL_SIZE_FORMAT " X %" CPL_SIZE_FORMAT
+                                      " (%s)", im1->nx, im1->ny,
+                                      cpl_type_get_name(im1->type), im2->nx,
+                                      im2->ny, cpl_type_get_name(im2->type));
+    } else if (im1->ny != im2->ny) {
+        return cpl_error_set_message_(CPL_ERROR_INCOMPATIBLE_INPUT,
+                                      "Images are %" CPL_SIZE_FORMAT " X %"
+                                      CPL_SIZE_FORMAT " (%s) <=> %"
+                                      CPL_SIZE_FORMAT " X %" CPL_SIZE_FORMAT
+                                      " (%s)", im1->nx, im1->ny,
+                                      cpl_type_get_name(im1->type), im2->nx,
+                                      im2->ny, cpl_type_get_name(im2->type));
+    }
 
     assert( im1->pixels );
     assert( im2->pixels );
@@ -1116,7 +1307,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
           size_t       i;
 
           for (i = 0; i < nxy; i++)
-              pio[i] = (CPL_TYPE)CPL_MATH_ABS((CPL_MATH_TYPE)pio[i]);
+              pio[i] = (CPL_TYPE)CPL_MATH_ABS(pio[i]);
 #ifdef CPL_TYPE_IS_FPOINT
           cpl_tools_add_flops( image->nx * image->ny );
 #endif
@@ -1577,7 +1768,6 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 #undef CPL_TYPE_CONCAT
 #undef CPL_TYPE_T
 #undef CPL_MATH_ABS
-#undef CPL_MATH_TYPE
 
 #undef CPL_TYPE_ADD
 #undef CPL_TYPE_IS_FPOINT
