@@ -1,4 +1,4 @@
-/* $Id: cpl_image_io_body.h,v 1.58 2012/03/19 12:42:54 llundin Exp $
+/* $Id: cpl_image_io_body.h,v 1.59 2013-04-03 13:27:52 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -154,7 +154,7 @@
 #if defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_GET
     case CPL_TYPE_T: {
         const CPL_TYPE * pi = (CPL_TYPE*)image->pixels;
-        value = (double)pi[pos];
+        value = (CPL_TYPE_RETURN)pi[pos];
         break;
     }
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_SET
