@@ -1,4 +1,4 @@
-/* $Id: cpl_image_io_impl.h,v 1.14 2013/02/20 15:21:16 llundin Exp $
+/* $Id: cpl_image_io_impl.h,v 1.15 2013-04-12 07:49:36 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2013/02/20 15:21:16 $
- * $Revision: 1.14 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-04-12 07:49:36 $
+ * $Revision: 1.15 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifndef CPL_IMAGE_IO_IMPL_H
@@ -66,7 +66,7 @@ cpl_image * cpl_image_load_(fitsfile *, int *, CPL_FITSIO_TYPE[], cpl_type *,
                             cpl_boolean, cpl_size, cpl_size, cpl_size, cpl_size)
     CPL_ATTR_ALLOC;
 
-cpl_error_code cpl_image_save_(const cpl_image **, cpl_size, cpl_boolean,
+cpl_error_code cpl_image_save_(const cpl_image * const*, cpl_size, cpl_boolean,
                                const char *, cpl_type,
                                const cpl_propertylist *, unsigned)
     CPL_INTERNAL;
