@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic_body.h,v 1.42 2012/06/13 15:14:42 llundin Exp $
+/* $Id: cpl_imagelist_basic_body.h,v 1.43 2013-10-09 13:25:29 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -210,7 +210,9 @@
                                 ngood[i] = 0;
                             } else {
                                 po[i + j * mx] = (CPL_TYPE)0;
-                                cpl_image_reject(median, i + 1, j + 1);
+                                cpl_image_reject(median,
+                                                 (i + j * mx) % nx + 1,
+                                                 (i + j * mx) / nx + 1);
                             }
                         }
                     }
