@@ -1,4 +1,4 @@
-/* $Id: cpl_tools_body.h,v 1.28 2012/08/23 15:17:43 cgarcia Exp $
+/* $Id: cpl_tools_body.h,v 1.33 2013-05-28 08:33:57 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cgarcia $
- * $Date: 2012/08/23 15:17:43 $
- * $Revision: 1.28 $
- * $Name: cpl-6_3_1 $
+ * $Author: llundin $
+ * $Date: 2013-05-28 08:33:57 $
+ * $Revision: 1.33 $
+ * $Name: not supported by cvs2svn $
  */
 
 #undef ADDTYPE
@@ -46,6 +46,9 @@ static void ADDTYPE(cpl_tools_get_median_9)(CPL_TYPE *) CPL_ATTR_NONNULL;
 
 static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE *) CPL_ATTR_NONNULL;
 
+static void ADDTYPE(cpl_tools_get_0th)(CPL_TYPE *, size_t) CPL_ATTR_NONNULL;
+
+
 /* Swap macro */
 #define CPL_TYPE_SWAP(a, b) { register const CPL_TYPE t=(a); (a)=(b); (b)=t; }
 
@@ -53,6 +56,37 @@ static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE *) CPL_ATTR_NONNULL;
 /* Guarded swap macro */
 #define CPL_TYPE_SORT(a, b) { if ((a) > (b)) CPL_TYPE_SWAP((a), (b)); }
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Get the smallest value in a numerical array
+  @param  self The array to permute and request from
+  @param  n    The number of elements in the array, must be at least 1
+  @return void
+  @see cpl_tools_get_kth
+  @note Since the function is static its error checking is disabled
+  @note cpl_tools_get_0th(self, n) is the same as
+  (void)cpl_tools_get_kth(self, n, 0)
+
+ */
+/*----------------------------------------------------------------------------*/
+static void ADDTYPE(cpl_tools_get_0th)(CPL_TYPE * self, size_t n)
+{
+    CPL_TYPE min = self[0];
+    size_t amin = 0;
+    size_t i;
+
+    for (i = 1; i < n; i++) {
+        if (self[i] < min) {
+            amin = i;
+            min = self[i];
+        }
+    }
+
+    CPL_TYPE_SWAP(self[0], self[amin]);
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -68,18 +102,18 @@ static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE *) CPL_ATTR_NONNULL;
   For an even number of elements in the array, the mean of the two central
   values is returned. Note that in this case, the median might not be a value
   of the input array. Also, note that in the case of integer data types, 
-  the result will be converted to an integer.
-  Consider to transform your int array to float if that is not the desired 
+  the division by 2 is performed with integer arithmetic.
+  Consider casting your integer array to float if that is not the desired 
   behavior.  
 
-  The function is optimized for 3, 5, 6, 9 and 25 elements.
+  The function is optimized for 3, 5, 6, 7, 9 and 25 elements.
 
   After a successful call, self is permuted so elements less than the median
   have lower indices, while elements greater than the median have higher
   indices.
 
   Example: the median of (1 2 3) is 2 and the median of (1. 2. 3. 4.) is 2.5.
-  The median of integers (1 2 3 4) is 2 (since it is implicitly converted).
+  The median of the integers (1 2 3 4) is 2 (due to the integer arithmetic).
  */
 /*----------------------------------------------------------------------------*/
 CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, cpl_size n)
@@ -111,19 +145,19 @@ CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, cpl_size n)
         break;
 
     default:
-        if(n & 1)  //Odd number
+        if (n & 1) {
+            /* Odd number */
             (void)ADDTYPE(cpl_tools_quickselection)(self, n, (n-1)/2);
-        else      //Even number
-        {
+        } else {
+            /* Even number */
             (void)ADDTYPE(cpl_tools_quickselection)(self, n, n/2-1);
-            (void)ADDTYPE(cpl_tools_quickselection)(self + n/2, n - n/2, 0);
+            ADDTYPE(cpl_tools_get_0th)(self + n / 2, n / 2);
         }
     }
 
-    if(n & 1)  //Odd number
-        return self[(n-1)/2];
-    else      //Even number
-        return (self[n/2-1]+self[n/2])/((CPL_TYPE)2);
+    return (n & 1)
+        ? self[(n-1)/2] /* Odd number */
+        : (self[n/2-1]+self[n/2])/((CPL_TYPE)2); /* Even number */
 }
 
 
@@ -383,14 +417,17 @@ static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE * self)
 
  */
 /*----------------------------------------------------------------------------*/
-double ADDTYPE(cpl_tools_get_mean)(const CPL_TYPE * a, cpl_size n)
+double ADDTYPE(cpl_tools_get_mean)(const CPL_TYPE * a, cpl_size nn)
 {
     double   mean = 0.0;
-    cpl_size i;
+    const size_t n = (size_t)nn;
+    size_t i;
 
 
-    cpl_ensure(a != NULL, CPL_ERROR_NULL_INPUT,    0.0);
-    cpl_ensure(n >  0,    CPL_ERROR_ILLEGAL_INPUT, 0.0);
+    cpl_ensure(a  != NULL,       CPL_ERROR_NULL_INPUT,       0.0);
+    cpl_ensure(nn >  0,          CPL_ERROR_ILLEGAL_INPUT,    0.0);
+    /* Ensure that the cast was OK */
+    cpl_ensure((cpl_size)n == nn, CPL_ERROR_UNSUPPORTED_MODE, 0.0);
 
     for (i = 0; i < n; i++)
         mean += ((double)a[i] - mean) / (double)(i + 1);
@@ -487,14 +524,17 @@ $$\sum_{i=1}^{N+1} (x_i - \overline{x}_{n+1})2 = \sum_{i=1}^N (x_i - \overline{x
  */
 /*----------------------------------------------------------------------------*/
 double ADDTYPE(cpl_tools_get_variancesum)(const CPL_TYPE * a,
-                                          cpl_size n, double * pmean)
+                                          cpl_size nn, double * pmean)
 {
     double   varsum = 0.0;
     double   mean = 0.0;
-    cpl_size i;
+    const size_t n = (size_t)nn;
+    size_t i;
 
     cpl_ensure(a != NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(n >= 0, CPL_ERROR_ILLEGAL_INPUT, 0.0);
+    cpl_ensure(nn >= 0,   CPL_ERROR_ILLEGAL_INPUT, 0.0);
+    /* Ensure that the cast was OK */
+    cpl_ensure((cpl_size)n == nn, CPL_ERROR_UNSUPPORTED_MODE, 0.0);
 
     for (i=0; i < n; i++) {
         const double delta = (double)a[i] - mean;
@@ -585,7 +625,8 @@ static int ADDTYPE(compar_desc)(const void * p1, const void * p2)
   @internal
   @brief  Get the median of a numerical array
   @param  self The array to permute and request from
-  @param  n    The number of elements in the array
+  @param  nn    The number of elements in the array
+  @param  kk    The requested value position in the sorted array, zero for 1st
   @return The median of the partially sorted array.
   @see cpl_tools_get_median_int
   @note Since the function is not exported its error checking is disabled
@@ -602,26 +643,31 @@ static int ADDTYPE(compar_desc)(const void * p1, const void * p2)
  */
 /*----------------------------------------------------------------------------*/
 CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
-                                           cpl_size   n,
-                                           cpl_size   k)
+                                           cpl_size   nn,
+                                           cpl_size   kk)
 {
 
-    cpl_size       low  = 0;
-    cpl_size       high = n-1;
+    const size_t n = (size_t)nn;
+    const size_t k = (size_t)kk;
+    size_t       low  = 0;
+    size_t       high = n-1;
 
 #ifdef CPL_TOOLS_STRICT_ERROR_CHECKING
     cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT,    (CPL_TYPE)0);
-    cpl_ensure(n >  0,       CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
-    cpl_ensure(n >  k -1,    CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
+    cpl_ensure(nn  >  0,     CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
+    cpl_ensure(nn  >  kk -1, CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
 #endif
+    /* Ensure that the cast was OK */
+    cpl_ensure((cpl_size)n == nn, CPL_ERROR_UNSUPPORTED_MODE, 0.0);
+    cpl_ensure((cpl_size)k == kk, CPL_ERROR_UNSUPPORTED_MODE, 0.0);
 
     /* Control flow has been changed to a single return at the end */
 
     for (;low + 1 < high;) {
         /* Find median of low, middle and high items; swap into position low */
-        const cpl_size middle = (low + high) / 2;
-        cpl_size       ll = low + 1;
-        cpl_size       hh = high;
+        const size_t middle = low + (high - low) / 2;
+        size_t       ll = low + 1;
+        size_t       hh = high;
 
         CPL_TYPE_SORT(self[middle], self[high]);
         CPL_TYPE_SORT(self[low],    self[high]);
@@ -670,7 +716,7 @@ CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
   @see cpl_tools_get_median_int
 
   After a successful call, self is permuted so elements less than the kth have
-  lower indices, while elements greather than the kth have higher indices.
+  lower indices, while elements greater than the kth have higher indices.
 
   Reference:
 
@@ -769,7 +815,7 @@ void ADDTYPE(cpl_tools_sort_ascn)(CPL_TYPE * self, int n)
 /*----------------------------------------------------------------------------*/
 void ADDTYPE(cpl_tools_sort_desc)(CPL_TYPE * self, int n)
 {
-    qsort(self, n, sizeof(*self), ADDTYPE(compar_desc));
+    qsort(self, (size_t)n, sizeof(*self), ADDTYPE(compar_desc));
 }
 
 /*----------------------------------------------------------------------------*/
@@ -789,12 +835,16 @@ void ADDTYPE(cpl_tools_sort_desc)(CPL_TYPE * self, int n)
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
-ADDTYPE(cpl_tools_sort)(CPL_TYPE * self, cpl_size n)
+ADDTYPE(cpl_tools_sort)(CPL_TYPE * self, cpl_size nn)
 {
-    cpl_size    i, ir, j, k, l;
-    cpl_size    i_stack[CPL_PIX_STACK_SIZE];
-    cpl_size    j_stack;
-    CPL_TYPE    a;
+    const size_t n = (size_t)nn;
+    size_t    i, ir, j, k, l;
+    size_t    i_stack[CPL_PIX_STACK_SIZE];
+    size_t    j_stack;
+    CPL_TYPE  a;
+
+    /* Ensure that the cast was OK */
+    cpl_ensure_code((cpl_size)n == nn, CPL_ERROR_UNSUPPORTED_MODE);
 
     ir = n;
     l = 1;
