@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic-test.c,v 1.62 2012/06/12 07:40:57 llundin Exp $
+/* $Id: cpl_imagelist_basic-test.c,v 1.63 2013-10-09 13:25:29 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/06/12 07:40:57 $
- * $Revision: 1.62 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-10-09 13:25:29 $
+ * $Revision: 1.63 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -73,6 +73,7 @@ void cpl_imagelist_collapse_sigclip_create_test_one(const cpl_imagelist *);
 
 void cpl_imagelist_collapse_create_test(cpl_type);
 void cpl_imagelist_collapse_sigclip_create_test(void);
+static void cpl_imagelist_collapse_median_create_test_one(void);
 
 /*-----------------------------------------------------------------------------
                                   Main
@@ -101,6 +102,7 @@ int main(void)
 
     do_bench = cpl_msg_get_level() <= CPL_MSG_INFO ? CPL_TRUE : CPL_FALSE;
 
+    cpl_imagelist_collapse_median_create_test_one();
     cpl_imagelist_collapse_create_test(CPL_TYPE_INT);
     cpl_imagelist_collapse_create_test(CPL_TYPE_FLOAT);
     cpl_imagelist_collapse_create_test(CPL_TYPE_DOUBLE);
@@ -520,7 +522,7 @@ static void cpl_imagelist_collapse_median_create_tests(cpl_size mz,
 
         const cpl_type type = pixel_type[itype];
         cpl_imagelist * imglist = cpl_imagelist_new();
-        cpl_image     * image;
+        cpl_image     * image = NULL;
         cpl_size        i;
         cpl_error_code code;
 
@@ -830,7 +832,7 @@ static void cpl_imagelist_collapse_sigclip_create_bench(cpl_size          mz,
     const cpl_size  nz = mz;
     const cpl_size  nx = 2*IMAGESZBIG;
     const cpl_size  ny = 2*IMAGESZBIG;
-    cpl_image     * image;
+    cpl_image     * image = NULL;
     cpl_image     * map = cpl_image_new(nx, ny, CPL_TYPE_INT);
     cpl_error_code error;
     double          secs;
@@ -943,8 +945,6 @@ void cpl_imagelist_collapse_create_test(cpl_type type)
 
 }
 
-
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Test the CPL function
@@ -1200,3 +1200,32 @@ void cpl_imagelist_collapse_sigclip_create_test(void)
     cpl_image_delete(map);
 
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Test the CPL function
+  @return   void
+  @note This test activates the cache optimization given a 18MB L2 cache
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_imagelist_collapse_median_create_test_one(void)
+{
+
+    cpl_imagelist * imlist = cpl_imagelist_new();
+    cpl_image * img;
+    cpl_size i;
+
+    for (i = 0 ; i < 300; i++) {
+        img = cpl_image_new(IMAGESZBIG, IMAGESZBIG/2, CPL_TYPE_FLOAT);
+        cpl_image_reject(img, IMAGESZBIG, IMAGESZBIG/2);
+        cpl_imagelist_set(imlist, img, i);
+    }
+    img = cpl_imagelist_collapse_median_create(imlist);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(img);
+
+    cpl_test(cpl_image_is_rejected(img, IMAGESZBIG, IMAGESZBIG/2));
+
+    cpl_image_delete(img);
+    cpl_imagelist_delete(imlist);
+}
