@@ -1,4 +1,4 @@
-/* $Id: cpl_mask-test.c,v 1.71 2011/11/22 14:50:30 llundin Exp $
+/* $Id: cpl_mask-test.c,v 1.72 2013-10-14 12:14:13 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/11/22 14:50:30 $
- * $Revision: 1.71 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-10-14 12:14:13 $
+ * $Revision: 1.72 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -66,6 +66,7 @@ static void cpl_mask_filter_test(int);
 static void cpl_mask_filter_test_schalkoff(void);
 static void cpl_mask_filter_test_matrix(int);
 static void cpl_mask_filter_bench(int, int, int, int, int, int);
+static void cpl_mask_count_bench(int, int);
 
 static cpl_error_code cpl_mask_closing_matrix(cpl_mask *, const cpl_matrix *);
 static cpl_error_code cpl_mask_opening_matrix(cpl_mask *, const cpl_matrix *);
@@ -129,6 +130,11 @@ int main(void)
 
     cpl_test_nonnull( stream );
 
+    if (do_bench) {
+        cpl_mask_count_bench(1024, 1000);
+    } else {
+        cpl_mask_count_bench(10, 1);
+    }
     cpl_mask_filter_test_schalkoff();
     cpl_mask_filter_test(IMAGESZ);
 
@@ -1751,6 +1757,48 @@ static void cpl_mask_filter_test_matrix(int nsize) {
 }
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief       Benchmark the CPL function
+  @param nsize The object size to use for testing
+  @param nreps The number of repetitions
+  @return      void
+  
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_mask_count_bench(int nsize, int nreps) {
+
+    cpl_mask * mask1 = cpl_mask_new(nsize, nsize);
+    cpl_mask * mask2;
+    cpl_error_code error;
+    double time0, time_mask = 0.0;
+    int irep;
+
+    error = cpl_mask_fill_test(mask1, 2.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    mask2 = cpl_mask_duplicate(mask1);
+    error = cpl_mask_not(mask2);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    time0 = cpl_test_get_cputime();
+
+    for (irep = 0; irep < nreps; irep++) {
+
+        cpl_test_eq(cpl_mask_count(mask1) + cpl_mask_count(mask2),
+                    nsize * nsize);
+    }
+
+    time_mask = cpl_test_get_cputime() - time0;
+
+    cpl_msg_info(cpl_func, "Time to count elements 2 X %d mask of size %d x %d "
+                 "[s]: %g", nreps, nsize, nsize, time_mask);
+    cpl_mask_delete(mask1);
+    cpl_mask_delete(mask2);
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
