@@ -1,4 +1,4 @@
-/* $Id: cpl_matrix-test.c,v 1.57 2012/11/16 09:52:52 llundin Exp $
+/* $Id: cpl_matrix-test.c,v 1.59 2013-09-05 11:21:15 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/11/16 09:52:52 $
- * $Revision: 1.57 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-09-05 11:21:15 $
+ * $Revision: 1.59 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -201,6 +201,7 @@ int main(void)
     const double * data;
     cpl_boolean  do_bench;
     cpl_error_code code;
+    const cpl_matrix * null; /* This one is expected to be NULL */
 
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
@@ -439,6 +440,25 @@ int main(void)
 
   cpl_matrix_delete(copia);
 
+  /* Test error handling for invalid input: */
+  null = cpl_matrix_extract_row(NULL, 1);
+  cpl_test_error(CPL_ERROR_NULL_INPUT);
+  cpl_test_null(null);
+  null = cpl_matrix_extract_row(matrix, 9999999);
+  cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+  cpl_test_null(null);
+  null = cpl_matrix_extract_row(matrix, -1);
+  cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+  cpl_test_null(null);
+  null = cpl_matrix_extract_column(NULL, 1);
+  cpl_test_error(CPL_ERROR_NULL_INPUT);
+  cpl_test_null(null);
+  null = cpl_matrix_extract_column(matrix, 9999999);
+  cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+  cpl_test_null(null);
+  null = cpl_matrix_extract_column(matrix, -1);
+  cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+  cpl_test_null(null);
   test_data(copia, cpl_matrix_extract_row(matrix, 2), "Copy one row... ");
 
   c = cpl_matrix_get_ncol(matrix);
@@ -1182,7 +1202,7 @@ int main(void)
 
   for (i = 0; i < r; i++) {
       sprintf(message, 
-              "Check element %" CPL_SIZE_FORMAT ", %" CPL_SIZE_FORMAT
+              "Check element %" CPL_SIZE_FORMAT ", %d"
               " of horizontally merged matrix... ",
               i, 0);
       test_fvalue(cpl_matrix_get(product, i, 0), 0.00001,
@@ -1468,8 +1488,9 @@ int main(void)
   cpl_matrix_dump(copia, stream);
 
   cpl_msg_info("", "Try to solve a 2 X 2 singular system");
-  cpl_test_null( cpl_matrix_solve(matrix, copia));
+  null =  cpl_matrix_solve(matrix, copia);
   cpl_test_error(CPL_ERROR_SINGULAR_MATRIX);
+  cpl_test_null(null);
 
   cpl_matrix_delete(product);
   xtrue = cpl_matrix_new(1,1);
@@ -2043,7 +2064,7 @@ void cpl_matrix_product_bilinear_test(cpl_size nr, cpl_size nc,
     /* Same number of FP operations */
     cpl_test_eq(ffunc, fbrute);
 
-    cpl_test_matrix_abs(self, brute, DBL_EPSILON);
+    cpl_test_matrix_abs(self, brute, 250.0 * DBL_EPSILON);
 
     cpl_matrix_delete(self);
     cpl_matrix_delete(jacobi);
