@@ -1,4 +1,4 @@
-/* $Id: cpl_msg-test.c,v 1.14 2012/11/15 09:36:07 llundin Exp $
+/* $Id: cpl_msg-test.c,v 1.14 2012-11-15 09:36:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/11/15 09:36:07 $
+ * $Date: 2012-11-15 09:36:07 $
  * $Revision: 1.14 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #include <string.h>
@@ -45,7 +45,10 @@ int main(void)
 {
     char message[STR_LENGTH+1];
     char toolong[CPL_MAX_MSG_LENGTH];
+    char* verylongname = NULL;
+    int verylongnamesize = 1024*1024;
     int nindent = 0;
+    cpl_msg_severity loglevel = CPL_MSG_OFF;
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
@@ -88,6 +91,33 @@ int main(void)
     for (;nindent;  nindent--)
         cpl_msg_indent_less(); /* Undo the indenting */
 
+    /* Construct a very long null-terminated string of 'A' characters */
+    verylongname = (char*) malloc(verylongnamesize);
+    cpl_test_nonnull(verylongname);
+    (void)memset((void*)verylongname, 'A', verylongnamesize-1);
+    verylongname[verylongnamesize-1] = '\0';
+    
+    loglevel = cpl_msg_get_level();
+    cpl_msg_set_level(CPL_MSG_OFF);
+    cpl_msg_debug(verylongname, "test for long component name");
+    cpl_msg_set_level(loglevel);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_msg_set_level(CPL_MSG_OFF);
+    cpl_msg_info(verylongname, "test for long component name");
+    cpl_msg_set_level(loglevel);
+    cpl_msg_set_level(loglevel);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_msg_set_level(CPL_MSG_OFF);
+    cpl_msg_warning(verylongname, "test for long component name");
+    cpl_msg_set_level(loglevel);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_msg_set_level(CPL_MSG_OFF);
+    cpl_msg_error(verylongname, "test for long component name");
+    cpl_msg_set_level(loglevel);
+    cpl_test_error(CPL_ERROR_NONE);
+    
+    free(verylongname);
+
     /* End of tests */
     return cpl_test_end(0);
 
