@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs.c,v 1.136 2012/12/04 08:06:30 llundin Exp $
+/* $Id: cpl_dfs.c,v 1.138 2013-10-21 07:57:46 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/12/04 08:06:30 $
- * $Revision: 1.136 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-10-21 07:57:46 $
+ * $Revision: 1.138 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -552,6 +552,7 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     const char       *datamd5;
     const cpl_frame  *frame;
     const cpl_frame  *first_frame = NULL;
+    cpl_frameset_iterator *it = NULL;
     cpl_propertylist *plist;
     const cpl_parameter *param;
     int               nraw;
@@ -605,54 +606,122 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
      * Forbidden keywords are removed from product header.
      */
 
-    cpl_propertylist_erase_regexp(header, "^ESO DPR |^ARCFILE$|^ORIGFILE$", 0);
+    cpl_propertylist_erase_regexp(header,
+                                  "^ESO DPR |^ARCFILE$|^ORIGFILE$|"
+                                  "^CHECKSUM$|^DATASUM$", 0);
 
     /*
      * Get the first input frame in the input set-of-frames.
      */
 
-    frame = cpl_frameset_get_first_const(framelist);
+    it = cpl_frameset_iterator_new(framelist);
+    frame = cpl_frameset_iterator_get_const(it);
 
     while (frame != NULL) {
+
+    	cpl_errorstate status;
+
         if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_RAW) {
             first_frame = frame;
             break;
         }
-        frame = cpl_frameset_get_next_const(framelist);
+
+        status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+        	cpl_errorstate_set(status);
+        }
+
+        frame = cpl_frameset_iterator_get_const(it);
+
     }
 
     if (first_frame == NULL) {
-        frame = cpl_frameset_get_first_const(framelist);
+
+    	cpl_frameset_iterator_reset(it);
+        frame = cpl_frameset_iterator_get_const(it);
 
         while (frame != NULL) {
-            if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_CALIB) {
+
+        	cpl_errorstate status;
+
+        	if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_CALIB) {
                 first_frame = frame;
                 break;
             }
-            frame = cpl_frameset_get_next_const(framelist);
+
+            status = cpl_errorstate_get();
+
+            cpl_frameset_iterator_advance(it, 1);
+
+            if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+            	cpl_errorstate_set(status);
+            }
+
+            frame = cpl_frameset_iterator_get_const(it);
+
         }
     }
 
+	cpl_frameset_iterator_delete(it);
+	it = NULL;
+
     if (first_frame == NULL) {
         return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND,
                                      "Empty set-of-frames");
     }
 
     if (inherit_frame) {
-        int found = 0;
+        int iframe = 0;
 
-        frame = cpl_frameset_get_first_const(framelist);
-
-        while (frame != NULL) {
-            if (strcmp(cpl_frame_get_filename(frame), 
-                       cpl_frame_get_filename(inherit_frame)) == 0) {
-                found = 1;
-                break;
-            }
-            frame = cpl_frameset_get_next_const(framelist);
+        if (cpl_frame_get_filename(inherit_frame) == NULL) {
+            return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND,
+                                          "Inherit frame has no filename");
         }
 
-        if (found) {
+        it = cpl_frameset_iterator_new(framelist);
+        frame = cpl_frameset_iterator_get_const(it);
+
+        while (frame != NULL) {
+
+        	cpl_errorstate status;
+
+            iframe++;
+
+            if (cpl_frame_get_filename(frame) == NULL) {
+
+            	cpl_frameset_iterator_delete(it);
+
+                return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND, "Frame "
+                                              "%d/%d has no filename", iframe,
+                                              (int)cpl_frameset_get_size
+                                              (framelist));
+
+            }
+
+            if (strcmp(cpl_frame_get_filename(frame), 
+                       cpl_frame_get_filename(inherit_frame)) == 0) {
+                break;
+            }
+
+            status = cpl_errorstate_get();
+
+            cpl_frameset_iterator_advance(it, 1);
+
+            if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+            	cpl_errorstate_set(status);
+            }
+
+            frame = cpl_frameset_iterator_get_const(it);
+
+        }
+
+        cpl_frameset_iterator_delete(it);
+        it = NULL;
+
+        if (frame != NULL) {
             switch (cpl_is_fits(cpl_frame_get_filename(inherit_frame))) {
             case 0:
                 return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT,
@@ -896,9 +965,14 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
      */
 
     nraw = 0;
-    frame = cpl_frameset_get_first_const(framelist);
+
+    it = cpl_frameset_iterator_new(framelist);
+    frame = cpl_frameset_iterator_get_const(it);
 
     while (frame != NULL) {
+
+    	cpl_errorstate status;
+
         if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_RAW) {
 
             ++nraw;
@@ -916,9 +990,22 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
                                          "Category of raw frame");
 
         }
-        frame = cpl_frameset_get_next_const(framelist);
+
+        status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+        	cpl_errorstate_set(status);
+        }
+
+        frame = cpl_frameset_iterator_get_const(it);
+
     }
 
+    cpl_frameset_iterator_delete(it);
+    it = NULL;
+
 
     /*
      * PRO DATANCOM
@@ -936,9 +1023,14 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
      */
 
     ncal = 0;
-    frame = cpl_frameset_get_first_const(framelist);
+
+    it = cpl_frameset_iterator_new(framelist);
+    frame = cpl_frameset_iterator_get_const(it);
 
     while (frame != NULL) {
+
+    	cpl_errorstate status;
+
         if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_CALIB) {
 
             ++ncal;
@@ -969,9 +1061,11 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
                 cpl_ensure_code(plist != NULL, cpl_error_get_code());
                 break;
             default:
+            	cpl_frameset_iterator_delete(it);
                 return cpl_error_get_code() == CPL_ERROR_NULL_INPUT
                     ? cpl_error_set_(CPL_ERROR_DATA_NOT_FOUND)
                     : cpl_error_set_where_();
+                break;
             }
 
             if (cpl_propertylist_has(plist, DATAMD5)) {
@@ -989,9 +1083,23 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
             }
             cpl_propertylist_delete(plist);
         }
-        frame = cpl_frameset_get_next_const(framelist);
+
+        status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+        	cpl_errorstate_set(status);
+        }
+
+        frame = cpl_frameset_iterator_get_const(it);
+
     }
 
+    cpl_frameset_iterator_delete(it);
+    it = NULL;
+
+
     npar = 0;
     param = cpl_parameterlist_get_first_const(parlist);
 
@@ -1133,18 +1241,30 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
 {
     const cpl_frame * frame;
 
+    cpl_frameset_iterator *it;
 
     cpl_ensure_code(self != NULL, CPL_ERROR_NULL_INPUT);
 
-    for (frame = cpl_frameset_get_first_const(self);
-         frame != NULL;
-         frame = cpl_frameset_get_next_const(self)) {
+
+    it = cpl_frameset_iterator_new(self);
+
+    while ((frame = cpl_frameset_iterator_get_const(it)) != NULL) {
+
+    	cpl_errorstate status;
+
 
         if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_PRODUCT) {
 
             const char * filename = cpl_frame_get_filename(frame);
 
-            cpl_ensure_code(filename != NULL, CPL_ERROR_DATA_NOT_FOUND);
+            if (filename == NULL) {
+
+            	cpl_frameset_iterator_delete(it);
+
+            	cpl_error_set(cpl_func, CPL_ERROR_DATA_NOT_FOUND);
+            	return CPL_ERROR_DATA_NOT_FOUND;
+
+            }
 
             if (cpl_is_fits(filename)) {
 
@@ -1156,9 +1276,13 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
 
                 if (cpl_io_fits_open_diskfile(&fproduct, filename, READWRITE,
                                               &error)) {
+
+                	cpl_frameset_iterator_delete(it);
+
                     return cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
                                               fits_open_diskfile,
                                               "filename='%s'", filename);
+
                 }
 
 #ifndef CPL_IO_FITS_REWIND
@@ -1176,7 +1300,8 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                                               fits_read_card, "filename='%s', "
                                               "key='" DATAMD5 "'", filename);
                     error = 0;
-                } else {
+                }
+                else {
 
                     /* The fits header has the MD5-card, update it */
                     char * md5sum = cpl_dfs_find_md5sum(fproduct);
@@ -1184,17 +1309,9 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                     if (!md5sum) {
                         code = cpl_error_set_where_();
                     } else {
-                        /* fits_update_key_str() is missing const modifiers on
-                           the value and comment string arguments!
-                           fits_update_key_str() is trusted not to actually
-                           modify these strings
-                           - if that fails the behavior is undefined
-                           (e.g. segmentation violation).
-                           :-(((((((((((((((((((((((((((((((((((((((((((((( */
 
                         if (fits_update_key_str(fproduct, DATAMD5, md5sum,
-                                                (char*)"MD5 checksum",
-                                                &error)) {
+                                                "MD5 checksum", &error)) {
                             code = cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                                       fits_update_key_str,
                                                       "filename='%s', key='"
@@ -1202,8 +1319,8 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                                                       filename, md5sum);
                             error = 0;
                         } else if (fits_update_key_str(fproduct, PIPEFILE,
-                                                       (char*)filename,
-                                                       (char*)"Filename of data"
+                                                       filename,
+                                                       "Filename of data"
                                                        " product", &error)) {
                             code = cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                                       fits_update_key_str,
@@ -1216,15 +1333,35 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                 }
 
                 if (cpl_io_fits_close_file(fproduct, &error)) {
+
+                    cpl_frameset_iterator_delete(it);
+
                     return cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                               fits_close_file,
                                               "filename='%s'", filename);
+
+                }
+
+                if (code) {
+                    cpl_frameset_iterator_delete(it);
+                    return code;
                 }
-                if (code) return code;
             }
         }
+
+        status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+        	cpl_errorstate_set(status);
+        }
+
     }
 
+    cpl_frameset_iterator_delete(it);
+    it = NULL;
+
     return CPL_ERROR_NONE;
 
 }
@@ -1235,7 +1372,8 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
   @brief  Compute the MD5 hash of the data units in a FITS file.
   @param  fproduct A CFITSIO FITS file structure
   @return CPL_ERROR_NONE or the relevant CPL error code on error
-  @note   Upon success the CFITSIO FITS file structure is reset to the 1st HDU
+  @note   Upon success the created string must be deallocated with cpl_free()
+    and the CFITSIO FITS file structure is reset to the 1st HDU
 
  */
 /*----------------------------------------------------------------------------*/
