@@ -1,4 +1,4 @@
-/* $Id: md5.c,v 1.6 2010/11/11 09:23:18 llundin Exp $
+/* $Id: md5.c,v 1.7 2013-07-22 14:55:50 llundin Exp $
  *
  * This file is part of the ESO CPL Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.6 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-07-22 14:55:50 $
+ * $Revision: 1.7 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -37,14 +37,30 @@
                                Private Function prototypes
  -----------------------------------------------------------------------------*/
 
-static void MD5Transform(word32 buf[4], word32 const in[16]);
+static void MD5Transform(word32 buf[4], word32 const in[16])
+    CPL_ATTR_NONNULL;
 
 #if defined(WORDS_BIGENDIAN) && WORDS_BIGENDIAN != 0
-inline static void byteReverse(unsigned char *buf, unsigned longs);
+inline static void byteReverse(unsigned char * restrict buf, unsigned longs)
+    CPL_ATTR_NONNULL;
 #else
 #define byteReverse(BUF, LONGS) /* Do nothing */
 #endif
 
+/* This code is in the public domain */
+
+/* Each function has been redeclared so:
+   1) each array argument is now a restricted pointer
+   2) the nonnull attribute is used when available
+   3) it is static (the whole md5.c code is included in the single
+      source code file where it is used)
+
+   And finally in MD5Final():
+   4) the assignments causing a strict-aliasing warning have been rewritten
+   5) The widely noticed sizeof bug in the "In case it's sensitive" memset()
+      has been fixed
+*/
+
 /*----------------------------------------------------------------------------*/
 /**
  *                      MD5 message-digest algorithm
@@ -70,7 +86,7 @@ inline static void byteReverse(unsigned char *buf, unsigned longs);
  * Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
  * initialization constants.
  */
-static void MD5Init(struct MD5Context *ctx)
+static void MD5Init(struct MD5Context * restrict ctx)
 {
     ctx->buf[0] = 0x67452301;
     ctx->buf[1] = 0xefcdab89;
@@ -85,7 +101,8 @@ static void MD5Init(struct MD5Context *ctx)
  * Update context to reflect the concatenation of another buffer full
  * of bytes.
  */
-static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned len)
+static void MD5Update(struct MD5Context * restrict ctx,
+                      unsigned char const * restrict buf, unsigned len)
 {
     register word32 t;
 
@@ -105,10 +122,10 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
 
     t = 64 - t;
     if (len < t) {
-        memmove(p, buf, len);
+        memcpy(p, buf, len);
         return;
     }
-    memmove(p, buf, t);
+    memcpy(p, buf, t);
     byteReverse(ctx->in, 16);
     MD5Transform(ctx->buf, (word32 *) ctx->in);
     buf += t;
@@ -117,7 +134,7 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
     /* Process data in 64-byte chunks */
 
     while (len >= 64) {
-    memmove(ctx->in, buf, 64);
+    memcpy(ctx->in, buf, 64);
     byteReverse(ctx->in, 16);
     MD5Transform(ctx->buf, (word32 *) ctx->in);
     buf += 64;
@@ -126,14 +143,15 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
 
     /* Handle any remaining bytes of data. */
 
-    memmove(ctx->in, buf, len);
+    memcpy(ctx->in, buf, len);
 }
 
 /*
  * Final wrapup - pad to 64-byte boundary with the bit pattern 
  * 1 0* (64-bit count of bits processed, MSB-first)
  */
-static void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
+static void MD5Final(unsigned char * restrict digest,
+                     struct MD5Context * restrict ctx)
 {
     unsigned int count;
     unsigned char *p;
@@ -165,13 +183,16 @@ static void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
     byteReverse(ctx->in, 14);
 
     /* Append length in bits and transform */
-    ((word32 *) ctx->in)[14] = ctx->bits[0];
-    ((word32 *) ctx->in)[15] = ctx->bits[1];
+    /* Change the original assigments to avoid the gcc-4.8.1 warning:
+       dereferencing type-punned pointer will break strict-aliasing rules
+       [-Werror=strict-aliasing]
+    */
+    (void)memcpy(ctx->in + 56, ctx->bits, 8);
 
     MD5Transform(ctx->buf, (word32 *) ctx->in);
     byteReverse((unsigned char *) ctx->buf, 4);
-    memmove(digest, ctx->buf, 16);
-    memset(ctx, 0, sizeof(ctx));    /* In case it's sensitive */
+    memcpy(digest, ctx->buf, 16);
+    memset(ctx, 0, sizeof(*ctx));    /* In case it's sensitive */
 }
 
 /* The four core functions - F1 is optimized somewhat */
@@ -275,10 +296,7 @@ static void MD5Transform(word32 buf[4], word32 const in[16])
 }
 
 #if defined(WORDS_BIGENDIAN) && WORDS_BIGENDIAN != 0
-/*
- * Note: this code is harmless on little-endian machines.
- */
-inline static void byteReverse(unsigned char *buf, unsigned longs)
+inline static void byteReverse(unsigned char * restrict buf, unsigned longs)
 {
     word32 t;
     do {
