@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs-test.c,v 1.62 2012/04/05 08:48:50 llundin Exp $
+/* $Id: cpl_dfs-test.c,v 1.64 2013-10-21 07:57:47 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/04/05 08:48:50 $
- * $Revision: 1.62 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-10-21 07:57:47 $
+ * $Revision: 1.64 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -71,12 +71,23 @@ static void cpl_dfs_parameterlist_fill(cpl_parameterlist *);
 int main(void)
 {
 
+    cpl_propertylist * plist = NULL;
+    cpl_error_code     code;
+
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
     cpl_dfs_save_txt(CPL_DFS_RAW_ASCII);
 
+    plist = cpl_propertylist_new();
+
+    /* This card is of floating point type, but make sure it is written
+       as an integer, since that format is also valid */
+    code = cpl_propertylist_append_int(plist, "EQUINOX", 2000);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
     cpl_image_save(NULL, "inherit.fits",
-           CPL_TYPE_FLOAT, NULL, CPL_IO_CREATE);
+           CPL_TYPE_FLOAT, plist, CPL_IO_CREATE);
+    cpl_propertylist_delete(plist);
 
     /* Insert tests here */
 
@@ -97,6 +108,7 @@ static void cpl_dfs_product_tests(const char * rawname)
     cpl_frame         * rawframe;
     cpl_frame         * inhframe;
     cpl_frame         * proframe;
+    cpl_frame         * emptyframe;
     cpl_frameset      * frameset = NULL;
     cpl_propertylist  * plist = NULL;
     cpl_parameterlist * parlist = NULL;
@@ -127,6 +139,21 @@ static void cpl_dfs_product_tests(const char * rawname)
     plist = cpl_propertylist_new();
     parlist = cpl_parameterlist_new();
 
+
+    /* Insert CHECKSUM and DATASUM (with wrong values) */
+    error = cpl_propertylist_append_string(plist, "CHECKSUM",
+                                           "BADAKIZUEUSKARAZ");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_propertylist_set_comment(plist, "CHECKSUM", "HDU checksum");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_propertylist_append_string(plist, "DATASUM",
+                                           "3141592653");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_propertylist_set_comment(plist, "DATASUM",
+                                         "data unit checksum");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
     cpl_dfs_parameterlist_fill(parlist);
 
     error = cpl_dfs_setup_product_header(plist, proframe, frameset, parlist,
@@ -224,6 +251,9 @@ static void cpl_dfs_product_tests(const char * rawname)
                                          "PRO-1.15", NULL);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
+    cpl_test_zero(cpl_propertylist_has(plist, "CHECKSUM"));
+    cpl_test_zero(cpl_propertylist_has(plist, "DATASUM"));
+
     cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 
@@ -237,17 +267,27 @@ static void cpl_dfs_product_tests(const char * rawname)
     error = cpl_frameset_insert(frameset, inhframe);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
+    /* This card is of floating point type, but make sure it is written
+       as an integer, since that format is also valid */
+    cpl_propertylist_append_int(plist, "EQUINOX", 2000);
+
     error = cpl_dfs_setup_product_header(plist, proframe, frameset,
                                          parlist, "Recipe", "Pipeline",
                                          "PRO-1.15", inhframe);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
+    cpl_test_zero(cpl_propertylist_has(plist, "CHECKSUM"));
+    cpl_test_zero(cpl_propertylist_has(plist, "DATASUM"));
+
     cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 
     cpl_propertylist_append_char(plist, "ESO QC MYCHAR", 42);
     cpl_propertylist_set_comment(plist, "ESO QC MYCHAR", "42");
 
+    cpl_propertylist_append_string(plist, "ARCFILE",
+                                   "IIINSTRUME.2013-08-28T12:50:43.3381");
+
     error = cpl_dfs_save_paf("IIINSTRUME", "RRRECIPE", plist,
                              "cpl_dfs-test.paf");
     cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -281,6 +321,15 @@ static void cpl_dfs_product_tests(const char * rawname)
                                          "PRO-1.15", proframe);
 
     cpl_test_eq_error( error, CPL_ERROR_FILE_NOT_FOUND);
+    
+    /* Test 5c: Check error handling when given an empty frame with no
+       filename set. */
+    emptyframe = cpl_frame_new();
+    error = cpl_dfs_setup_product_header(plist, proframe, frameset,
+                                         parlist, "Recipe", "Pipeline",
+                                         "PRO-1.15", emptyframe);
+    cpl_test_eq_error(error, CPL_ERROR_DATA_NOT_FOUND);
+    cpl_frame_delete(emptyframe);
 
     /* Test 6a: A failure on file format (missing file) */
 
@@ -307,12 +356,15 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     cpl_test_error(error);
 
-
     /* Test 7a: A successful call of cpl_dfs_update_product_header()
        with an empty data-unit */
 
+    /* This card is of floating point type, but make sure it is written
+       as an integer, since that format is also valid */
+    cpl_propertylist_append_int(plist, "EQUINOX", 2000);
+
     cpl_test_zero(cpl_image_save(NULL, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
-                                      plist, CPL_IO_CREATE));
+                                 plist, CPL_IO_CREATE));
     cpl_test_fits(CPL_DFS_FITSFILE);
 
     error = cpl_dfs_update_product_header(frameset);
@@ -325,6 +377,8 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     cpl_test_nonnull(plist);
 
+    cpl_test(cpl_propertylist_has(plist, "EQUINOX"));
+
     md5sum = cpl_propertylist_get_string(plist, "DATAMD5");
 
     cpl_test_nonnull(md5sum);
@@ -339,6 +393,12 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     image = cpl_image_new(2, 3, CPL_TYPE_INT);
 
+    /* This card is of floating point type, but make sure it is written
+       as an integer, since that format is also valid */
+    cpl_propertylist_append_int(plist, "EQUINOX", 2000);
+
+    cpl_test(cpl_propertylist_has(plist, "EQUINOX"));
+
     cpl_test_zero(cpl_image_save(image, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
                                       plist, CPL_IO_CREATE));
     cpl_test_fits(CPL_DFS_FITSFILE);
@@ -408,6 +468,8 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     cpl_test_nonnull(plist);
 
+    cpl_test(cpl_propertylist_has(plist, "EQUINOX"));
+
     md5sum = cpl_propertylist_get_string(plist, "DATAMD5");
 
     cpl_test_nonnull(md5sum);
@@ -481,15 +543,19 @@ static void cpl_dfs_product_tests(const char * rawname)
            work-around :-( */
 
         cpl_frameset * newframes = cpl_frameset_new();
-        const cpl_frame * frame;
         const char * newformat = "newname_%04d.fits";
         cpl_frame * newframe;
 
-        /* Create new frameset of products with new filenames
-           - and rename product files */
-        for (frame = cpl_frameset_get_first(frameset), i = 0;
-             frame != NULL;
-             frame = cpl_frameset_get_next(frameset)) {
+        cpl_frameset_iterator *it = cpl_frameset_iterator_new(frameset);
+        const cpl_frame *frame = cpl_frameset_iterator_get_const(it);
+
+        /*
+         *  Create new frameset of products with new filenames
+         *  - and rename product files
+         */
+
+        i = 0;
+        while (frame != NULL) {
 
             if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_PRODUCT) {
 
@@ -514,8 +580,22 @@ static void cpl_dfs_product_tests(const char * rawname)
 
                 }
             }
+
+            cpl_errorstate status = cpl_errorstate_get();
+
+            cpl_frameset_iterator_advance(it, 1);
+
+            if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+                cpl_errorstate_set(status);
+            }
+
+            frame = cpl_frameset_iterator_get_const(it);
+
         }
 
+        cpl_frameset_iterator_delete(it);
+        it = NULL;
+
         cpl_msg_info("", "Renamed %d product(s)", i);
         error = cpl_dfs_update_product_header(newframes);
         cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -597,6 +677,11 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     cpl_dfs_parameterlist_fill(parlist);
 
+    /* This card is of floating point type, but make sure it is written
+       as an integer, since that format is also valid */
+    error = cpl_propertylist_append_int(qclist, "EQUINOX", 2000);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
     cpl_test_zero(cpl_propertylist_append_string(qclist, CPL_DFS_PRO_CATG,
                                                  "procat"));
 
