@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures.c,v 1.34 2012/11/14 12:23:29 llundin Exp $
+/* $Id: cpl_apertures.c,v 1.34 2012-11-14 12:23:29 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/11/14 12:23:29 $
+ * $Date: 2012-11-14 12:23:29 $
  * $Revision: 1.34 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1400,7 +1400,7 @@ cpl_apertures * cpl_apertures_extract_window(const cpl_image  * self,
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Simple apertures creation from a user supplied selction mask
+  @brief    Simple apertures creation from a user supplied selection mask
   @param    self    The image to process
   @param    selection  The mask of selected pixels
   @return   The list of detected apertures or NULL if nothing detected or on
