@@ -1,4 +1,4 @@
-/* $Id: cpl_fit.c,v 1.46 2012/06/15 14:26:56 llundin Exp $
+/* $Id: cpl_fit.c,v 1.46 2012-06-15 14:26:56 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/06/15 14:26:56 $
+ * $Date: 2012-06-15 14:26:56 $
  * $Revision: 1.46 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1865,6 +1865,61 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
 }
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Evaluate the Gaussian in a 2D-point
+  @param    self  The seven Gaussian parameters
+  @param    x     The X-coordinate to evaluate
+  @param    y     The Y-coordinate to evaluate
+  @return   The gaussian value or zero on error
+  @see cpl_fit_image_gaussian()
+  @note The function should not be able to fail if the parameters come from
+        a succesful call to cpl_fit_image_gaussian()
+
+   Possible #_cpl_error_code_ set in this function:
+   - CPL_ERROR_NULL_INPUT if a pointer is NULL.
+   - CPL_ERROR_TYPE_MISMATCH if the array is not of type double
+   - CPL_ERROR_ILLEGAL_INPUT if the array has a length different from 7
+   - CPL_ERROR_ILLEGAL_OUTPUT if the (absolute value of the) radius exceeds 1
+   - CPL_ERROR_DIVISION_BY_ZERO if a sigma is 0, or the radius is 1
+
+ */
+/*----------------------------------------------------------------------------*/
+double cpl_gaussian_eval_2d(const cpl_array * self, double x, double y)
+{
+    cpl_errorstate prestate = cpl_errorstate_get();
+    const double B    = cpl_array_get_double(self, 0, NULL);
+    const double A    = cpl_array_get_double(self, 1, NULL);
+    const double R    = cpl_array_get_double(self, 2, NULL);
+    const double M_x  = cpl_array_get_double(self, 3, NULL);
+    const double M_y  = cpl_array_get_double(self, 4, NULL);
+    const double S_x  = cpl_array_get_double(self, 5, NULL);
+    const double S_y  = cpl_array_get_double(self, 6, NULL);
+
+    double value = 0.0;
+
+    if (!cpl_errorstate_is_equal(prestate)) {
+        (void)cpl_error_set_where_();
+    } else if (cpl_array_get_size(self) != 7) {
+        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+    } else if (fabs(R) < 1.0 && S_x != 0.0 && S_y != 0.0) {
+        const double x_n  = (x - M_x) / S_x;
+        const double y_n  = (y - M_y) / S_y;
+
+        value = B + A / (CPL_MATH_2PI * S_x * S_y * sqrt(1 - R * R)) *
+            exp(-0.5 / (1 - R * R) * ( x_n * x_n + y_n * y_n
+                                       - 2.0 * R * x_n * y_n));
+    } else if (fabs(R) > 1.0) {
+        (void)cpl_error_set_message_(CPL_ERROR_ILLEGAL_OUTPUT,
+                                     "fabs(R=%g) > 1", R);
+    } else {
+        (void)cpl_error_set_message_(CPL_ERROR_DIVISION_BY_ZERO,
+                                     "R=%g. Sigma=(%g, %g)", R, S_x, S_y);
+    }
+
+    return value;
+}
+
 /**@}*/
 
 /*----------------------------------------------------------------------------*/
