@@ -1,4 +1,4 @@
-/* $Id: cpl_wcs.c,v 1.54 2013/01/07 12:39:11 llundin Exp $
+/* $Id: cpl_wcs.c,v 1.55 2013-04-26 13:44:02 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2013/01/07 12:39:11 $
- * $Revision: 1.54 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-04-26 13:44:02 $
+ * $Revision: 1.55 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,23 +34,57 @@
                                    Includes
  ----------------------------------------------------------------------------*/
 
-#include <cpl_memory.h>
-#include <cpl_tools.h>
-#include <cpl_error_impl.h>
-#include <cpl_propertylist_impl.h>
-#include <cpl_math_const.h>
-#include <cpl_errorstate.h>
-#include "cpl_wcs.h"
 
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
 #include <fitsio.h>
 
+#include <cxstring.h>
+
+#include <cpl_memory.h>
+#include <cpl_tools.h>
+#include <cpl_error_impl.h>
+#include <cpl_propertylist_impl.h>
+#include <cpl_math_const.h>
+#include <cpl_errorstate.h>
+
+#include "cpl_wcs.h"
+
+
+
 #ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
+
+/*
+ * Undefine PACKAGE_ symbols to avoid redefinition warnings from the compiler.
+ * Although these symbols are meant for internal use wcslib exports them in
+ * as part of a public header.
+ */
+
+#ifdef PACKAGE_NAME
+#  undef PACKAGE_NAME
+#endif
+
+#ifdef PACKAGE_VERSION
+#  undef PACKAGE_VERSION
+#endif
+
+#ifdef PACKAGE_TARNAME
+#  undef PACKAGE_TARNAME
+#endif
+
+#ifdef PACKAGE_STRING
+#  undef PACKAGE_STRING
+#endif
+
+#ifdef PACKAGE_BUGREPORT
+#  undef PACKAGE_BUGREPORT
+#endif
+
 #include <wcslib.h>
 #endif                      /* End If WCS is installed */
 
+
 /*---------------------------------------------------------------------------*/
 /**
  * @defgroup cpl_wcs World Coordinate System
@@ -84,6 +118,8 @@ struct _cpl_wcs_ {
     cpl_array     *imdims;     /* Wrapper for image dimensions */
     cpl_array     *crval;     /* CRVALia keyvalues for each coord axis */
     cpl_array     *crpix;     /* CRPIXja keyvalues for each pixel axis */
+    cpl_array     *ctype;     /* CTYPEja keyvalues for each pixel axis */
+    cpl_array     *cunit;     /* CUNITja keyvalues for each pixel axis */
     cpl_matrix    *cd;         /* CDi_ja linear transformation matrix */
 };
 
@@ -104,6 +140,7 @@ static void cpl_wcs_platesol_4(const cpl_matrix *xy, const cpl_matrix *std,
 static void cpl_wcs_platesol_6(const cpl_matrix *xy, const cpl_matrix *std,
                                const cpl_array *bad, cpl_array **plateconsts)
     CPL_ATTR_NONNULL;
+static int _cpl_wcsset(cpl_wcs *wcs);
 
 #ifdef fits_write_hdu
   /* fits_write_hdu() was introduced in CFITSIO version 3.03 */
@@ -256,12 +293,40 @@ cpl_wcs *cpl_wcs_new_from_propertylist(const cpl_propertylist *plist) {
 
     if (wcs->naxis > 0) {
 
+        int k = 0;
+
         /* If this is a table, then just copy the vital WCS information over
            and leave the image dimensional information as NULL */
 
         wcs->crval = cpl_array_wrap_double(wcs->wcsptr->crval, wcs->naxis);
         wcs->crpix = cpl_array_wrap_double(wcs->wcsptr->crpix, wcs->naxis);
-        wcs->cd    = cpl_matrix_wrap(wcs->naxis, wcs->naxis, wcs->wcsptr->cd);
+        wcs->cd    = cpl_matrix_wrap(wcs->naxis, wcs->naxis,
+                                     wcs->wcsptr->lin.piximg);
+
+        /*
+         * Make the contents of CUNITia and CTYPEia available through
+         * accessors.
+         *
+         * Due to the way wcslib stores this information, wrapping
+         * wcsptr->cunit and wcsptr->ctype is not possible, and therefore
+         * the elements have to be copied.
+         *
+         * NOTE: The elements of wcsptr->cuint and wcsptr->ctype get updated
+         * when wcsset() is called. Thus the copy has to be updated after each
+         * call to wcsset()!
+         *
+         * Apart from this function where the structure is initially created
+         * use _cpl_wcsset() to make sure that the copied fields get properly
+         * updated
+         */
+
+        wcs->ctype = cpl_array_new(wcs->naxis, CPL_TYPE_STRING);
+        wcs->cunit = cpl_array_new(wcs->naxis, CPL_TYPE_STRING);
+
+        for (k = 0; k < wcs->naxis; ++k) {
+            cpl_array_set_string(wcs->ctype, k, wcs->wcsptr->ctype[k]);
+            cpl_array_set_string(wcs->cunit, k, wcs->wcsptr->cunit[k]);
+        }
 
         if (! istab ) {
             cpl_errorstate prevstate = cpl_errorstate_get();
@@ -308,6 +373,8 @@ cpl_wcs *cpl_wcs_new_from_propertylist(const cpl_propertylist *plist) {
         /* FIXME: Is it correct/necessary to set to NULL ? */
         wcs->crval = NULL;
         wcs->crpix = NULL;
+        wcs->ctype = NULL;
+        wcs->cunit = NULL;
         wcs->cd    = NULL;
     }
 
@@ -357,6 +424,10 @@ void cpl_wcs_delete(cpl_wcs *wcs) {
     }
     cpl_array_unwrap(wcs->crval);
     cpl_array_unwrap(wcs->crpix);
+
+    cpl_array_delete(wcs->ctype);
+    cpl_array_delete(wcs->cunit);
+
     cpl_matrix_unwrap(wcs->cd);
 
     if (wcs->wcsptr != NULL) {
@@ -873,6 +944,8 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
     (wp->csyer)[0] = mederr_xi;
     (wp->csyer)[1] = mederr_eta;
 
+    _cpl_wcsset(wcs);
+
     /* Make a FITS string and convert it to a propertylist */
 
     if (wcshdo(0,wp, &i, &o)) {
@@ -893,22 +966,49 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
         *olist = cpl_wcs_fitsstr2plist(outstr);
         cpl_free(outstr);
 
-        /* Ok, massage the result a bit to make it look a bit nicer */
 
-        p = cpl_propertylist_get_property(*olist,"PC1_1");
-        cpl_property_set_name(p,"CD1_1");
-        p = cpl_propertylist_get_property(*olist,"PC1_2");
-        cpl_property_set_name(p,"CD1_2");
-        p = cpl_propertylist_get_property(*olist,"PC2_1");
-        cpl_property_set_name(p,"CD2_1");
-        p = cpl_propertylist_get_property(*olist,"PC2_2");
-        cpl_property_set_name(p,"CD2_2");
+        for (i = 2; i > 0; --i) {
+
+            unsigned int j;
+
+            for (j = 2; j > 0; --j)
+            {
+                cx_string *cd = cx_string_new();
+
+                cx_string_sprintf(cd, "CD%1d_%1d", i, j);
+                cpl_property *_p = cpl_property_new(cx_string_get(cd),
+                                                    CPL_TYPE_DOUBLE);
+
+                cpl_property_set_double(_p,
+                                        (wp->lin.piximg)[(i - 1) * 2 + (j - 1)]);
+                cpl_property_set_comment(_p, "Coordinate transformation matrix "
+                                         "element");
+
+                cx_string_delete(cd);
+
+                cpl_propertylist_insert_after_property(*olist, "CRPIX2", _p);
+                cpl_property_delete(_p);
+
+            }
+
+        }
+
+
+        /*
+         * Remove keywords which are not needed for the DICB required
+         * CD representation of the linear transformation matrix.
+         */
+
+        // FIXME: What about the RESTFRQ, RESTWAV stuff? Needed or not?
+
+        cpl_propertylist_erase_regexp(*olist, "^PC[12]_[12]$", 0);
         cpl_propertylist_erase(*olist,"CDELT1");
         cpl_propertylist_erase(*olist,"CDELT2");
         cpl_propertylist_erase(*olist,"RESTFRQ");
         cpl_propertylist_erase(*olist,"RESTWAV");
         cpl_propertylist_erase(*olist,"END");
 
+
         /* Fix boo-boo in wcslib */
         /* The above uninformative comment means:
            Change type of PV-properties from int to double */
@@ -1198,6 +1298,98 @@ const cpl_array *cpl_wcs_get_crpix(const cpl_wcs *wcs) {
 #endif                      /* End If WCS is installed */
 }
 
+/**
+ * @brief
+ *    Accessor to get the CTYPE vector for a WCS
+ *
+ * @param wcs  The WCS structure to examine
+ *
+ * @return
+ *   A handle to an array with the CTYPEja keyvalues for each pixel axis,
+ *   or NULL on error.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>wcs</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NO_WCS</td>
+ *       <td class="ecr">
+ *         The WCS sub library is not available.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ * The function returns a handle to an array with the CTYPE vector defined for
+ * this WCS.
+ */
+
+const cpl_array *cpl_wcs_get_ctype(const cpl_wcs *wcs) {
+
+    /* Check for NULL input */
+    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
+
+#ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
+
+    return wcs->wcsptr->naxis == 0 ? NULL : wcs->ctype;
+
+#else
+    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
+#endif                      /* End If WCS is installed */
+}
+
+/**
+ * @brief
+ *    Accessor to get the CUNIT vector for a WCS
+ *
+ * @param wcs  The WCS structure to examine
+ *
+ * @return
+ *   A handle to an array with the CUNITja keyvalues for each pixel axis,
+ *   or NULL on error.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>wcs</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NO_WCS</td>
+ *       <td class="ecr">
+ *         The WCS sub library is not available.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ * The function returns a handle to an array with the CUNIT vector defined for
+ * this WCS.
+ */
+
+const cpl_array *cpl_wcs_get_cunit(const cpl_wcs *wcs) {
+
+    /* Check for NULL input */
+    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
+
+#ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
+
+    return wcs->wcsptr->naxis == 0 ? NULL : wcs->cunit;
+
+#else
+    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
+#endif                      /* End If WCS is installed */
+}
+
 /**
  * @brief
  *    Accessor to get the CD matrix for a WCS
@@ -1278,6 +1470,8 @@ static cpl_wcs *cpl_wcs_init(void) {
     self->imdims = NULL;
     self->crval = NULL;
     self->crpix = NULL;
+    self->ctype = NULL;
+    self->cunit = NULL;
     self->cd = NULL;
 
     return(self);
@@ -1411,12 +1605,10 @@ static cpl_propertylist *cpl_wcs_fitsstr2plist(const char *fitsstr) {
 
     /* Loop for all the cards in the FITS string and add them to the
        memory FITS header */
-    /* FIXME: fits_insert_card() is missing a const modifier */
-    while (strncmp(f, "END     ", 8) &&
-           !fits_insert_card(fptr, (char*)f, &ioerror)) {
+    while (strncmp(f, "END     ", 8) && !fits_insert_card(fptr, f, &ioerror)) {
         f += (FLEN_CARD - 1);
     }
-    if (ioerror || fits_insert_card(fptr, (char*)f, &ioerror)) {
+    if (ioerror || fits_insert_card(fptr, f, &ioerror)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, ioerror, fits_insert_card,
                                  "fitsstr='%s'", fitsstr);
         ioerror = 0;
@@ -1670,6 +1862,26 @@ static void cpl_wcs_platesol_4(const cpl_matrix *xy, const cpl_matrix *std,
     pc[5] = -xposmean*pc[3] - yposmean*pc[4] + etamean;
 }
 
+
+inline static int
+_cpl_wcsset(cpl_wcs *wcs)
+{
+    int k;
+    int status = wcsset(wcs->wcsptr);
+
+    /*
+     * Update indirectly, exported data which may have been updated by
+     * the call to wcsset().
+     */
+
+    for (k = 0; k < wcs->naxis; ++k) {
+        cpl_array_set_string(wcs->ctype, k, wcs->wcsptr->ctype[k]);
+        cpl_array_set_string(wcs->cunit, k, wcs->wcsptr->cunit[k]);
+    }
+
+    return status;
+
+}
 #ifndef fits_write_hdu
 
 /*-------------------------------------------------------------------------*/
