@@ -1,4 +1,4 @@
-/* $Id: cpl_frameset.c,v 1.44 2013/02/11 14:34:10 rpalsa Exp $
+/* $Id: cpl_frameset.c,v 1.45 2013-08-28 06:44:37 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: rpalsa $
- * $Date: 2013/02/11 14:34:10 $
- * $Revision: 1.44 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-08-28 06:44:37 $
+ * $Revision: 1.45 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -361,6 +361,76 @@ _cpl_frameset_history_sort(cx_list *self, const cx_multimap *frames,
 }
 
 
+inline static cpl_frame *
+_cpl_frameset_get_first(const cpl_frameset *self)
+{
+
+    cx_list_iterator first;
+    cx_multimap_iterator position;
+
+
+    first = cx_list_begin(self->history);
+    if (first == cx_list_end(self->history)) {
+        return NULL;
+    }
+
+    position = cx_list_get(self->history, first);
+    cx_assert(position != cx_multimap_end(self->frames));
+
+    /* Will modify internals of self! */
+    _cpl_frameset_cache_push(self, POS, position);
+
+    return cx_multimap_get_value(self->frames, position);
+
+}
+
+
+inline static cpl_frame *
+_cpl_frameset_get_next(const cpl_frameset *self)
+{
+
+    cx_list_iterator first;
+    cx_list_iterator last;
+
+    cx_multimap_iterator next;
+
+
+    cx_assert(_cpl_frameset_cache_get(self, POS) != NULL);
+
+
+    first = cx_list_begin(self->history);
+    last = cx_list_end(self->history);
+
+    while (first != last) {
+
+        next = cx_list_get(self->history, first);
+
+        if (next == _cpl_frameset_cache_get(self, POS)) {
+            break;
+        }
+
+        first = cx_list_next(self->history, first);
+
+    }
+
+    first =  cx_list_next(self->history, first);
+    if (first == cx_list_end(self->history)) {
+        return NULL;
+    }
+
+    next = cx_list_get(self->history, first);
+    if (next == cx_multimap_end(self->frames)) {
+        return NULL;
+    }
+
+    /* Will modify internals of self! */
+    _cpl_frameset_cache_push(self, POS, next);
+
+    return cx_multimap_get_value(self->frames, next);
+
+}
+
+
 /*
  * Public methods
  */
@@ -809,27 +879,12 @@ cpl_frameset_get_first_const(const cpl_frameset *self)
 
     const cxchar *const _id = "cpl_frameset_get_first_const";
 
-    cx_list_iterator first;
-    cx_multimap_iterator position;
-
-
     if (self == NULL) {
         cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
         return NULL;
     }
 
-    first = cx_list_begin(self->history);
-    if (first == cx_list_end(self->history)) {
-        return NULL;
-    }
-
-    position = cx_list_get(self->history, first);
-    cx_assert(position != cx_multimap_end(self->frames));
-
-    /* Will modify internals of self! */
-    _cpl_frameset_cache_push(self, POS, position);
-
-    return cx_multimap_get_value(self->frames, position);
+    return (const cpl_frame *)_cpl_frameset_get_first(self);
 
 }
 
@@ -870,13 +925,14 @@ cpl_frame *
 cpl_frameset_get_first(cpl_frameset *self)
 {
 
-    cpl_errorstate prestate = cpl_errorstate_get();
-    cpl_frame *frame = (cpl_frame *)cpl_frameset_get_first_const(self);
+    const cxchar *const _id = "cpl_frameset_get_first";
 
-    if (!cpl_errorstate_is_equal(prestate))
-        (void)cpl_error_set_where(cpl_func);
+    if (self == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
 
-    return frame;
+    return _cpl_frameset_get_first(self);
 
 }
 
@@ -923,52 +979,17 @@ cpl_frameset_get_next_const(const cpl_frameset *self)
 
     const cxchar *const _id = "cpl_frameset_get_next_const";
 
-    cx_list_iterator first;
-    cx_list_iterator last;
-
-    cx_multimap_iterator next;
-
 
     if (self == NULL) {
         cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
         return NULL;
     }
 
-    cx_assert(_cpl_frameset_cache_get(self, POS) != NULL);
-
-
-    first = cx_list_begin(self->history);
-    last = cx_list_end(self->history);
-
-    while (first != last) {
-
-        next = cx_list_get(self->history, first);
-
-        if (next == _cpl_frameset_cache_get(self, POS)) {
-            break;
-        }
-
-        first = cx_list_next(self->history, first);
-
-    }
-
-    first =  cx_list_next(self->history, first);
-    if (first == cx_list_end(self->history)) {
-        return NULL;
-    }
-
-    next = cx_list_get(self->history, first);
-    if (next == cx_multimap_end(self->frames)) {
-        return NULL;
-    }
-
-    /* Will modify internals of self! */
-    _cpl_frameset_cache_push(self, POS, next);
-
-    return cx_multimap_get_value(self->frames, next);
+    return (const cpl_frame *)_cpl_frameset_get_next(self);
 
 }
 
+
 /**
  * @brief
  *   Get the next frame in the given set.
@@ -1010,13 +1031,14 @@ cpl_frame *
 cpl_frameset_get_next(cpl_frameset *self)
 {
 
-    cpl_errorstate prestate = cpl_errorstate_get();
-    cpl_frame *frame = (cpl_frame *)cpl_frameset_get_next_const(self);
+    const cxchar *const _id = "cpl_frameset_get_next";
 
-    if (!cpl_errorstate_is_equal(prestate))
-        (void)cpl_error_set_where(cpl_func);
+    if (self == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
 
-    return frame;
+    return _cpl_frameset_get_next(self);
 
 }
 /**
@@ -1483,13 +1505,15 @@ cpl_frameset_labelise(const cpl_frameset *self,
     cpl_size *labels;
     cpl_size *labelsinv;
     cpl_size nframes;
-    cpl_size i;
-    cpl_size nlabels = 0;
+    cpl_size i = 0;
     cpl_size j = 0;
+    cpl_size nlabels = 0;
     cpl_size ncomp = 0;
     const cpl_frame ** framelist;
     const cpl_frame * frame1;
 
+    cpl_frameset_iterator *it = NULL;
+
 
     cpl_ensure(self      != NULL, CPL_ERROR_NULL_INPUT, NULL);
     cpl_ensure(compare   != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -1503,13 +1527,19 @@ cpl_frameset_labelise(const cpl_frameset *self,
     labelsinv = cx_malloc(nframes * sizeof(cpl_size));
     framelist = cx_malloc(nframes * sizeof(const cpl_frame *));
 
-    for (frame1 = cpl_frameset_get_first_const(self), i = 0; frame1 != NULL;
-         frame1 = cpl_frameset_get_next_const(self), i++) {
+    it = cpl_frameset_iterator_new(self);
+
+    while ((frame1 = cpl_frameset_iterator_get(it)) != NULL)
+    {
+
+    	cpl_errorstate status;
 
         cpl_size jj;
 
-        for (jj = 0; jj < nlabels; jj++, j = j == nlabels-1 ? 0 : j+1) {
-            const cpl_frame * frame2 = framelist[j];
+
+        for (jj = 0; jj < nlabels; jj++, j = (j == nlabels - 1) ? 0 : j + 1) {
+
+            const cpl_frame *frame2 = framelist[j];
 
             /*
              * Compare the frames i and j
@@ -1528,9 +1558,11 @@ cpl_frameset_labelise(const cpl_frameset *self,
             } else if (comp != 0) {
                 /* Error */
 
-                cx_free(labels);
+            	cpl_frameset_iterator_delete(it);
+
                 cx_free(framelist);
                 cx_free(labelsinv);
+                cx_free(labels);
 
                 /* Propagate error */
                 (void)cpl_error_set_where_();
@@ -1549,8 +1581,24 @@ cpl_frameset_labelise(const cpl_frameset *self,
             /* Identical */
             labels[i] = labels[labelsinv[j]];
         }
+
+
+    	status = cpl_errorstate_get();
+
+    	cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE)
+        {
+            cpl_errorstate_set(status);
+        }
+
+    	++i;
+
     }
 
+    cpl_frameset_iterator_delete(it);
+
+
     *nb_labels = nlabels;
     cx_free(framelist);
     cx_free(labelsinv);
@@ -1619,42 +1667,21 @@ cpl_frameset_get_frame_const(const cpl_frameset *set, cpl_size position)
 
     const cxchar *const _id = "cpl_frameset_get_frame_const";
 
-    cpl_size nbframes;
-    cpl_size i;
-
-    const cpl_frame *frame;
-
-
     if (set == NULL) {
         cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
         return NULL;
     }
 
-
-    /*
-     * Get the number of frames and check ind
-     */
-
-    nbframes = cpl_frameset_get_size(set);
-    if (position < 0 || position >= nbframes) {
+    if ((position < 0) || (position >= cpl_frameset_get_size(set))) {
         cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
         return NULL;
     }
 
-
-    /*
-     * Find the requested frame
-     */
-
-    frame = cpl_frameset_get_first_const(set);
-    for (i = 0; i < position; i++) {
-        frame = cpl_frameset_get_next_const(set);
-    }
-
-    return frame;
+    return cpl_frameset_get_position_const(set, position);
 
 }
 
+
 /**
  * @brief
  *   Get a frame from a frame set.
@@ -1709,13 +1736,19 @@ cpl_frame *
 cpl_frameset_get_frame(cpl_frameset *set, cpl_size position)
 {
 
-    cpl_errorstate prestate = cpl_errorstate_get();
-    cpl_frame *frame = (cpl_frame *)cpl_frameset_get_frame_const(set, position);
+    const cxchar *const _id = "cpl_frameset_get_frame";
 
-    if (!cpl_errorstate_is_equal(prestate))
-        (void)cpl_error_set_where(cpl_func);
+    if (set == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
 
-    return frame;
+    if ((position < 0) || (position >= cpl_frameset_get_size(set))) {
+        cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
+        return NULL;
+    }
+
+    return cpl_frameset_get_position(set, position);
 
 }
 
@@ -1775,6 +1808,9 @@ cpl_frameset_get_position(cpl_frameset *self, cpl_size position)
         return NULL;
     }
 
+
+    // FIXME: Should use iterators for that!
+
     _position = cx_list_begin(self->history);
 
     while (position > 0) {
@@ -1845,6 +1881,9 @@ cpl_frameset_get_position_const(const cpl_frameset *self, cpl_size position)
         return NULL;
     }
 
+
+    // FIXME: Should use iterators for that!
+
     _position = cx_list_begin(self->history);
 
     while (position > 0) {
@@ -1895,14 +1934,15 @@ cpl_frameset_extract(const cpl_frameset *self, const cpl_size *labels,
 
     cpl_frameset    *selected = NULL;
     const cpl_frame *frame;
-    cpl_size i;
+    cpl_size i = 0;
 
 
     cpl_ensure(self   != NULL, CPL_ERROR_NULL_INPUT, NULL);
     cpl_ensure(labels != NULL, CPL_ERROR_NULL_INPUT, NULL);
 
-    for (frame = cpl_frameset_get_first_const(self), i = 0; frame != NULL;
-         frame = cpl_frameset_get_next_const(self), i++) {
+    cpl_frameset_iterator *it = cpl_frameset_iterator_new(self);
+
+    while ((frame = cpl_frameset_iterator_get_const(it))) {
 
         if (labels[i] == desired_label) {
 
@@ -1910,16 +1950,32 @@ cpl_frameset_extract(const cpl_frameset *self, const cpl_size *labels,
              * Duplicate frame and insert it in the selected object
              */
 
-            cpl_frame * framecopy = cpl_frame_duplicate(frame);
+            cpl_frame *_frame = cpl_frame_duplicate(frame);
 
-            if (selected == NULL) selected = cpl_frameset_new();
+            if (selected == NULL) {
+                selected = cpl_frameset_new();
+            }
 
-            cpl_frameset_insert(selected, framecopy);
+            cpl_frameset_insert(selected, _frame);
 
         }
 
+
+        cpl_errorstate status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE)
+        {
+            cpl_errorstate_set(status);
+        }
+
+        ++i;
+
     }
 
+    cpl_frameset_iterator_delete(it);
+
     return selected;
 
 }
@@ -1958,20 +2014,627 @@ cpl_frameset_dump(const cpl_frameset *self, FILE *stream)
         return;
 
     fprintf (stream, "+++ Frameset at address: %p\n", (const void *)self);
-    frame = cpl_frameset_get_first_const(self);
 
-    for (i = 0; i < n; i++){
+    cpl_frameset_iterator *it = cpl_frameset_iterator_new(self);
 
-        fprintf (stream, "%3d  ",i);
+    while ((frame = cpl_frameset_iterator_get_const(it))) {
+
+        fprintf(stream, "%3d  ", i++);
         cpl_frame_dump(frame, stream);
 
-        frame = cpl_frameset_get_next_const(self);
+        cpl_errorstate status = cpl_errorstate_get();
 
-        if (frame == NULL)
-            return;
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE)
+        {
+            cpl_errorstate_set(status);
+        }
 
     }
 
+    cpl_frameset_iterator_delete(it);
+
     return;
 }
 /**@}*/
+
+
+/**
+ * @defgroup cpl_frameset_iterator Frame Set Iterators
+ * @ingroup cpl_frameset
+ *
+ * @brief
+ *   Iterator support for frame sets.
+ *
+ * Frame set iterators allow to access the contents of a frame set sequentially,
+ * in an ordered manner. An iterator is created for a particular frame set,
+ * and it stays bound to that frame set until it is destroyed. However multiple
+ * iterators can be defined for the same frame set.
+ *
+ * By default, the order of frames in a frame set is the order in which the
+ * individual frames have been inserted. However, a particular sorting order
+ * can be defined by sorting the frame set using a custom comparison function
+ * for frames.
+ *
+ * Frame set iterators are supposed to be short-lived objects, and it is not
+ * recommended to use them for keeping, or passing around references to the
+ * frame set contents. In particular, changing the contents of the underlying
+ * frame set by erasing, or inserting frames may invalidate all existing
+ * iterators.
+ */
+
+/**@{*/
+
+/*
+ * Frame set iterator type
+ */
+
+struct _cpl_frameset_iterator_
+{
+    const cpl_frameset *parent;
+    cx_list_iterator position;
+
+    cxint direction;
+};
+
+
+/**
+ * @brief
+ *   Create a new frame set iterator.
+ *
+ * @param parent  The frame set for which the iterator is created.
+ *
+ * @return
+ *   The newly allocated frame set iterator object, or @c NULL if an error
+ *   occurred.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>parent</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function creates a new iterator object bound to the frame set @em
+ * parent. The iterator is initialized such that it points to the beginning
+ * of @em parent.
+ *
+ * The beginning is defined by the current ordering defined for the frame set
+ * @em parent.
+ *
+ * @see cpl_frameset_sort()
+ */
+
+cpl_frameset_iterator *
+cpl_frameset_iterator_new(const cpl_frameset *parent)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_new";
+
+    cpl_frameset_iterator *self = NULL;
+
+
+    if (parent == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
+
+
+    self = cx_malloc(sizeof *self);
+
+    self->parent    = parent;
+    self->position  = cx_list_begin(parent->history);
+    self->direction = 0;
+
+    return self;
+
+}
+
+
+/**
+ * @brief
+ *   Create a frame set iterator from an existing frame set iterator.
+ *
+ * @param other  The frame set iterator to clone.
+ *
+ * @return
+ *   A copy of the frame set iterator @em other on success, or @c NULL
+ *   otherwise.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>other</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function creates a copy of the iterator object @em other. An iterator
+ * copy constructed by this function is bound to the same frame set @em other
+ * has been constructed for, and it points to the same frame that @em other
+ * points to.
+ */
+
+cpl_frameset_iterator *
+cpl_frameset_iterator_duplicate(const cpl_frameset_iterator *other)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_duplicate";
+
+    cpl_frameset_iterator *self = NULL;
+
+
+    if (other == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
+
+    self = cx_malloc(sizeof *self);
+
+    self->parent    = other->parent;
+    self->position  = other->position;
+    self->direction = other->direction;
+
+    return self;
+
+}
+
+
+/**
+ * @brief
+ *   Destroy a frame set iterator.
+ *
+ * @param self  The frame set to destroy.
+ *
+ * @return
+ *   Nothing.
+ *
+ * The function destroys the frame set iterator object @em self. If @em self
+ * is @c NULL, no operation is performed.
+ */
+
+void
+cpl_frameset_iterator_delete(cpl_frameset_iterator *self)
+{
+    cx_free(self);
+    return;
+}
+
+
+/**
+ * @brief
+ *   Assign a frame set iterator to another.
+ *
+ * @param self   The frame set iterator to assign to.
+ * @param other  The frame set iterator to be assigned.
+ *
+ * @return
+ *   The function returns @c CPL_ERROR_NONE on success, or an appropriate
+ *   CPL error code otherwise.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameters <i>self</i> or <i>other</i> is a <tt>NULL</tt>
+ *         pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_ILLEGAL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameters <i>self</i> and <i>other</i> are not bound to the
+ *         same frame set.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function assigns the iterator @em other to the iterator @em self.
+ * Only iterators which are bound to the same frame set can be assigned to
+ * each other!
+ */
+
+cpl_error_code
+cpl_frameset_iterator_assign(cpl_frameset_iterator *self,
+                             const cpl_frameset_iterator *other)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_assign";
+
+
+    if ((self == NULL) || (other == NULL)) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return CPL_ERROR_NULL_INPUT;
+    }
+
+    if (self->parent != other->parent) {
+        cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
+        return CPL_ERROR_ILLEGAL_INPUT;
+    }
+
+
+    self->position  = other->position;
+    self->direction = other->direction;
+
+    return CPL_ERROR_NONE;
+
+}
+
+
+/**
+ * @brief
+ *   Reset a frame set iterator to the beginning of a frame set.
+ *
+ * @param self  The iterator to reposition.
+ *
+ * @return
+ *   Nothing.
+ *
+ * The function moves the frame set iterator @em self back to the beginning
+ * of its underlying frame set. The first frame in the frame set is defined by
+ * the established sorting order.
+ *
+ * @see cpl_frameset_sort()
+ */
+
+void
+cpl_frameset_iterator_reset(cpl_frameset_iterator *self)
+{
+
+    if (self != NULL) {
+        self->position  = cx_list_begin(self->parent->history);
+        self->direction = 0;
+    }
+
+    return;
+
+}
+
+
+/**
+ * @brief
+ *   Advance an iterator by a number of elements.
+ *
+ * @param self      The frame set iterator to reposition.
+ * @param distance  The of number of elements by which the iterator is moved.
+ *
+ * @return
+ *   The function returns @c CPL_ERROR_NONE on success, or an appropriate
+ *   CPL error code otherwise.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_ACCESS_OUT_OF_RANGE</td>
+ *       <td class="ecr">
+ *         The given iterator offset would move the iterator beyond the
+ *         beginning or the end of the frame set.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The functions moves the iterator by @em distance number of elements, with
+ * respect to its current position. The number of elements @em distance may
+ * be negative or positive, and the iterator position will move backward and
+ * forward respectively.
+ *
+ * It is an error if the given @em distance would move the iterator either
+ * past the one-before-the-beginning position or the one-past-the-end position
+ * of the underlying frame set. In this case the iterator is not repositioned
+ * and an out of range error is returned.
+ */
+
+cpl_error_code
+cpl_frameset_iterator_advance(cpl_frameset_iterator *self, int distance)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_advance";
+
+    if (self == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return CPL_ERROR_NULL_INPUT;
+    }
+    else {
+
+        cxint _distance = distance;
+
+        cx_list_iterator position = self->position;
+
+
+        if (_distance < 0) {
+
+            if (self->direction == -1) {
+                position = cx_list_previous(self->parent->history, position);
+                ++_distance;
+            }
+
+            while ((_distance != 0) &&
+                   (position != cx_list_end(self->parent->history))) {
+                position = cx_list_previous(self->parent->history, position);
+                ++_distance;
+            }
+
+        }
+        else {
+
+            if (self->direction == 1) {
+                position = cx_list_begin(self->parent->history);
+                --_distance;
+            }
+
+            while ((_distance != 0) &&
+                   (position != cx_list_end(self->parent->history))) {
+                position = cx_list_next(self->parent->history, position);
+                --_distance;
+            }
+
+        }
+
+        if (_distance != 0) {
+
+            cpl_error_set(_id, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+            return CPL_ERROR_ACCESS_OUT_OF_RANGE;
+
+        }
+        else {
+
+            if (position == cx_list_end(self->parent->history)) {
+                self->direction = (distance < 0) ? 1 : -1;
+            }
+            else {
+                self->direction = 0;
+            }
+
+            self->position = position;
+        }
+
+    }
+
+    return CPL_ERROR_NONE;
+
+}
+
+
+/**
+ * @brief
+ *   Calculate the distance between two iterators
+ *
+ * @param self   First frame set iterator.
+ * @param other  Second frame set iterator.
+ *
+ * @return
+ *   The function returns the distance between the two input iterators. If an
+ *   error occurs, the function returns a distance of 0 and sets an
+ *   appropriate error code.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> or <i>other</i> is a <tt>NULL</tt>
+ *          pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_ILLEGAL_INPUT</td>
+ *       <td class="ecr">
+ *         The two iterators iterators are not bound to the same frame set.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_DATA_NOT_FOUND</td>
+ *       <td class="ecr">
+ *         At least one input iterator is invalid.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function calculates the distance between the iterators @em self and
+ * @em other.
+ *
+ * To properly detect whether this function has failed or not it is
+ * recommended to reset the errors before it is called, since the returned
+ * value is not a distinctive feature.
+ */
+
+int
+cpl_frameset_iterator_distance(const cpl_frameset_iterator *self,
+                               const cpl_frameset_iterator *other)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_distance";
+
+    int invalid = 0;
+
+
+    if ((self == NULL) || (other == NULL)) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return invalid;
+    }
+
+    if (self->parent != other->parent) {
+        cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
+        return invalid;
+    }
+
+
+    /*
+     * Nothing to be done if the iterator positions match already ...
+     */
+
+    if (self->position != other->position) {
+
+        /*
+         * ... otherwise try forward direction first ...
+         */
+
+        int distance = 0;
+        cx_list_iterator position = self->position;
+
+        while (position != cx_list_end(self->parent->history)) {
+
+            if (position == other->position) {
+                return distance;
+            }
+
+            ++distance;
+            position = cx_list_next(self->parent->history, position);
+
+        }
+
+
+        /*
+         * ... and finally the backward direction
+         */
+
+        distance = 0;
+        position = self->position;
+
+        while (position != cx_list_end(self->parent->history)) {
+
+            if (position == other->position) {
+                return distance;
+            }
+
+            --distance;
+            position = cx_list_previous(self->parent->history, position);
+
+        }
+
+
+        /*
+         * Nothing was found in any direction. This should not happen!
+         */
+
+        cpl_error_set(_id, CPL_ERROR_DATA_NOT_FOUND);
+        return invalid;
+
+    }
+
+    return 0;
+
+}
+
+
+/**
+ * @brief
+ *   Get the frame from the frame set at the current position of the iterator.
+ *
+ * @param self  The frame set iterator to dereference.
+ *
+ * @return
+ *   The frame stored in the frame set at the position of the iterator, or
+ *   @c NULL if an error occurs. The function also returns @cNULL if the
+ *   iterator is positioned at the sentinel element (i.e. the
+ *   one-before-the-beginning or one-past-the-end position).
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function dereferences the iterator @em self, i.e. it retrieves the frame
+ * at the position pointed to by @em self.
+ */
+
+cpl_frame *
+cpl_frameset_iterator_get(cpl_frameset_iterator *self)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_get";
+
+    cx_multimap_iterator position = NULL;
+
+
+    if (self == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
+
+    if (self->position == cx_list_end(self->parent->history)) {
+        return NULL;
+    }
+
+    position = cx_list_get(self->parent->history, self->position);
+    cx_assert(position != cx_multimap_end(self->parent->frames));
+
+    return cx_multimap_get_value(self->parent->frames, position);
+
+}
+
+
+/**
+ * @brief
+ *   Get the frame from the frame set at the current position of the iterator.
+ *
+ * @param self  The frame set iterator to dereference.
+ *
+ * @return
+ *   The frame stored in the frame set at the position of the iterator, or
+ *   @c NULL if an error occurs.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function dereferences the iterator @em self, i.e. it retrieves the frame
+ * at the position pointed to by @em self.
+ */
+
+const cpl_frame *
+cpl_frameset_iterator_get_const(const cpl_frameset_iterator *self)
+{
+
+    const cxchar *const _id = "cpl_frameset_iterator_get_const";
+
+    cx_multimap_iterator position = NULL;
+
+
+    if (self == NULL) {
+        cpl_error_set(_id, CPL_ERROR_NULL_INPUT);
+        return NULL;
+    }
+
+    if (self->position == cx_list_end(self->parent->history)) {
+        return NULL;
+    }
+
+    position = cx_list_get(self->parent->history, self->position);
+    cx_assert(position != cx_multimap_end(self->parent->frames));
+
+    return cx_multimap_get_value(self->parent->frames, position);
+
+}
+/**@}*/
