@@ -1,4 +1,4 @@
-/* $Id: cpl_frameset_io.c,v 1.34 2012/02/06 14:04:07 llundin Exp $
+/* $Id: cpl_frameset_io.c,v 1.34 2012-02-06 14:04:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/06 14:04:07 $
+ * $Date: 2012-02-06 14:04:07 $
  * $Revision: 1.34 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -94,6 +94,7 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
     const cpl_frame * cur_frame;
     cpl_size          selfsize;
 
+
     /* Test entries */
     cpl_ensure(fset  != NULL, CPL_ERROR_NULL_INPUT,    NULL);
     cpl_ensure(pnum  >= 0,    CPL_ERROR_ILLEGAL_INPUT, NULL);
@@ -103,21 +104,25 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
     self = cpl_imagelist_new();
     selfsize = 0;
 
-    for (cur_frame = cpl_frameset_get_first_const(fset); cur_frame != NULL;
-         cur_frame = cpl_frameset_get_next_const(fset)) {
+    cpl_frameset_iterator *it = cpl_frameset_iterator_new(fset);
+
+    while ((cur_frame = cpl_frameset_iterator_get_const(it)) != NULL) {
         const char * filename = cpl_frame_get_filename(cur_frame);
 
+        cpl_errorstate prestate;
+
         if (xtnum < 0) {
             /* Image(s) from all extensions requested.
              */
 
-            cpl_errorstate prestate = cpl_errorstate_get();
             int            nextensions, ixtnum;
             const cpl_size presize = selfsize;
 
             fitsfile     * fptr;
             int            error = 0;
 
+            prestate = cpl_errorstate_get();
+
             /* Initialize to indicate that they need to be read from the file */
             int             naxis = 0;
             CPL_FITSIO_TYPE naxes[3] ={0, 0, 0};
@@ -125,7 +130,12 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
 
             /* FIXME: Version 3.2 of fits_open_diskfile() seg-faults on NULL.
                If fixed in CFITSIO, this check should be removed */
-            cpl_ensure(filename != NULL, CPL_ERROR_NULL_INPUT, NULL);
+
+            if (filename == NULL) {
+                cpl_frameset_iterator_delete(it);
+                cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+                return NULL;
+            }
 
             if (cpl_io_fits_open_diskfile(&fptr, filename, READONLY, &error)) {
                 (void)cpl_error_set_fits(CPL_ERROR_ILLEGAL_INPUT, error,
@@ -209,8 +219,20 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
             if (cpl_imagelist_append_from_file(self, &selfsize, filename,
                                                im_type, pnum, xtnum)) break;
         }
+
+        prestate = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (!cpl_errorstate_is_equal(prestate)) {
+            if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+                cpl_errorstate_set(prestate);
+            }
+        }
+
     }
 
+
     if (cur_frame != NULL) {
         const char * filename = cpl_frame_get_filename(cur_frame);
         cpl_imagelist_delete(self);
@@ -219,6 +241,7 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
                                      ", xtnum=%" CPL_SIZE_FORMAT,
                                      filename ? filename : "<NULL>",
                                      (unsigned)im_type, pnum, xtnum);
+        cpl_frameset_iterator_delete(it);
         return NULL;
     }
 
@@ -233,10 +256,14 @@ cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset * fset,
                                      ", xtnum=%" CPL_SIZE_FORMAT,
                                      filename ? filename : "<NULL>",
                                      im_type, pnum, xtnum);
+        cpl_frameset_iterator_delete(it);
         return NULL;
     }
 
+    cpl_frameset_iterator_delete(it);
+
     return self;
+
 }
 
 /**@}*/
