@@ -1,4 +1,4 @@
-/* $Id: cxlist.c,v 1.8 2011/12/22 14:33:32 rpalsa Exp $
+/* $Id: cxlist.c,v 1.8 2011-12-22 14:33:32 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
  * Copyright (C) 2001-2011 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: rpalsa $
- * $Date: 2011/12/22 14:33:32 $
+ * $Date: 2011-12-22 14:33:32 $
  * $Revision: 1.8 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -817,6 +817,7 @@ cxsize
 cx_list_max_size(const cx_list *list)
 {
 
+    (void) list;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(list != NULL);
 
     return (cxsize)(-1);
@@ -887,6 +888,7 @@ cx_list_assign(cx_list *list, cx_list_iterator position, cxcptr data)
     cxptr tmp;
 
 
+    (void) list;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(list != NULL);
     cx_assert(_cx_lnode_exists(list, position));
 
@@ -970,6 +972,7 @@ cxptr
 cx_list_get(const cx_list *list, cx_list_const_iterator position)
 {
 
+    (void) list;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(list != NULL);
     cx_assert(_cx_lnode_exists(list, position));
 
@@ -999,6 +1002,7 @@ cx_list_insert(cx_list *list, cx_list_iterator position, cxcptr data)
 
     cx_lnode *node;
 
+    (void) list;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(list != NULL);
     cx_assert(position == _cx_list_end(list) ||
               _cx_lnode_exists(list, position));
@@ -1127,6 +1131,7 @@ cxptr
 cx_list_extract(cx_list *list, cx_list_iterator position)
 {
 
+    (void) list;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(list != NULL);
     cx_assert(_cx_lnode_exists(list, position));
 
@@ -1323,6 +1328,7 @@ cx_list_splice(cx_list *tlist, cx_list_iterator position,
                       cx_list_iterator last)
 {
 
+    (void)slist; (void)tlist;  /* Prevent warnings if cx_assert is disabled. */
     cx_assert(slist != NULL);
     cx_assert(first == _cx_list_end(slist) || _cx_lnode_exists(slist, first));
     cx_assert(last == _cx_list_end(slist) ||
